/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfio_f.c - Fortran interfaces to seismic benchmark file I/O
/*
/*  VERSION:
/*    Initial version, no support for different data types
/*
/*  Routines defined in this module:
/*    sbfcreat - create SBF file
/*    sbfopen - open SBF file
/*    sbfclose - close SBF file
/*    sbfnfiles - set number of files used for TRC data
/*    sbfqfiles - return number of files used for TRC data
/*    sbfpaths - set leading paths for multi-file SBF datasets
/*    sbfread - read traces and coordinates
/*    sbfwrite - write traces and coords 
/*    sbfreadt - read transposed traces and coords
/*    sbfwritet - write transposed traces and coords
/*    sbfreadp - read traces and coords in paneled mode
/*    sbfwritep - write traces and coords in paneled mode
/*    sbfrdfrm - read traces from SBF .TRC file
/*    sbfwrfrm - write traces to SBF .TRC file
/*    sbfrdprm - read parameters from SBF .HDR file
/*    sbfwrprm - write parameters to SBF .HDR file
/*    sbfqatr - query existance of attribute in .ATR file
/*    sbfrdatr - read attribute from SBF .ATR file
/*    sbfwratr - write attribute to SBF .ATR file
/*    sbfrinfo - read aux parameters from SBF .HDR file
/*    sbfwinfo - write aux parameters to SBF .HDR file
/*    sbfunlink - Permanently remove an SBF dataset
/*    sbferror - print SBF error string
/*    sbfsetvol - set volume number for subsequent reads/writes
/*     
/*--------------------------------------------------------------------*/
/*
/*  HISTORY:
/*
/*     Written:
/*       10/08/93 - C. Mosher, ARCO E&PT 
/*
/*     References:
/*
/*     Modified:
/*
/*--------------------------------------------------------------------*/

/* Structure containing field values */

#include "sbfio.h"
extern int sbf_error;

/* Fortran string allocators */
extern char *fstrnwcp(), *fstrncpy();

/* Array of handles - used to deal with 32/64 bit pointer issues */
#define MAXSBF 100
static SBFIO *hand[MAXSBF+1];
static int hindex = 1;
SBFIO *sbf_handle();

/* External names for Cray */
#ifdef CRAY
#define ISCRAY
#endif
#ifdef cray
#define ISCRAY
#endif

#ifdef ISCRAY
#define sbfcreat_ SBFCREAT
#define sbfopen_ SBFOPEN
#define sbfclose_ SBFCLOSE
#define sbfnfiles_ SBFNFILES
#define sbfqfiles_ SBFQFILES
#define sbfpaths_ SBFPATHS
#define sbfread_ SBFREAD
#define sbfwrite_ SBFWRITE
#define sbfreadt_ SBFREADT
#define sbfwritet_ SBFWRITET
#define sbfreadp_ SBFREADP
#define sbfwritep_ SBFWRITEP
#define sbfrdfrm_ SBFRDFRM
#define sbfwrfrm_ SBFWRFRM
#define sbfrdprm_ SBFRDPRM
#define sbfwrprm_ SBFWRPRM
#define sbfqatr_ SBFQATR
#define sbfrdatr_ SBFRDATR
#define sbfwratr_ SBFWRATR
#define sbfrinfo_ SBFRINFO
#define sbfwinfo_ SBFWINFO
#define sbfunlink_ SBFUNLINK
#define sbferror_ SBFERROR
#define sbfsetvol_ SBFSETVOL
#endif

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbf_handle - Dereference SBF pointer given handle
/*     
/*  SYNOPSIS:
/*    SBFIO *sbf, *sbf_handle();
/*    int *handle;
/*    sbf = sbf_handle( *handle );
/*
/*  DESCRIPTION:
/*    Returns SBF file pointer from the static handle array.
/*    Used to avoid problems passing 'int' vs 'long' pointers
/*    between Fortran and C.
/*
/*--------------------------------------------------------------------*/

SBFIO *sbf_handle( handle )
int handle;
{

/* Check for handle within range of currently allocated values */
	if (handle < 1 || handle >= hindex) 
		return((SBFIO *)NULL);
	else
		return( hand[handle] );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfcreat - Create SBF file
/*     
/*  SYNOPSIS:
/*    character*(*) path
/*    integer ns, ntpf, nfpv, nvol
/*    integer handle, sbfcreat
/*    handle = sbfcreat( path, ns, ntpf, nfpv, nvol )
/*
/*  DESCRTPTION
/*    Returns file handle for subsequent reads/writes to SBF dataset,
/*    zero on errors.
/*
/*--------------------------------------------------------------------*/

int sbfcreat_( path, ns, ntpf, nfpv, nvol, len_path )
char *path;
int *ns, *ntpf, *nfpv, *nvol;
int len_path;
{
	SBFIO *sbf;
	char *path_c;

/* Check for max files allocated */
	if (hindex > MAXSBF) return( 0 );

	path_c = fstrnwcp( path, len_path );
	sbf = sbf_creat( path_c, *ns, *ntpf, *nfpv, *nvol );
	free(path_c);

/* Exit if file open failed */
	if (sbf == (SBFIO *)NULL) return( 0 );

/* Store pointer in handle array, return index, increment */
	hand[hindex] = sbf;
	return( hindex++ );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfopen - Open SBF file
/*     
/*  SYNOPSIS:
/*    character*(*) path
/*    character*2 mode
/*    integer handle, sbfopen
/*    handle = sbfopen( path, mode )
/*
/*  DESCRIPTION:
/*    Returns a handle for reading/writing an existing SBF dataset
/*    Set 'mode' to 'rw' for read/write, 'r ' for read only
/*
/*--------------------------------------------------------------------*/

int sbfopen_( path, mode, len_path, len_mode )
char *path, *mode;
int len_path, len_mode;
{
	SBFIO *sbf;
	char *path_c;

/* Check for max files allocated */
	if (hindex > MAXSBF) return( 0 );
	path_c = fstrnwcp( path, len_path );
	sbf = sbf_open( path_c, mode );
	free(path_c);

/* Exit if file open failed */
	if (sbf == (SBFIO *)NULL) return(0);

/* Store pointer in handle array, return index, increment */
	hand[hindex] = sbf;
	return( hindex++ );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfclose - Close SBF dataset
/*     
/*  SYNOPSIS:
/*    integer ier, handle, sbfclose
/*    ier = sbfclose( handle )
/*
/*  DESCRIPTION:
/*    Closes an open SBF dataset
/*    Returns zero on success, -1 on errors
/*
/*--------------------------------------------------------------------*/

int sbfclose_( handle )
int *handle;
{
	SBFIO *sbf;

	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

	return( sbf_close( sbf ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfunlink - Permanently remove an SBF dataset
/*     
/*  SYNOPSIS:
/*    integer ier
/*    character*(*) path
/*    ier = sbfunlink( path )
/*
/*  DESCRIPTION:
/*    Permanently removes an SBF dataset from the file system
/*
/*--------------------------------------------------------------------*/

int sbfunlink_( path, len_path )
char *path;
int len_path;
{
	int iret;
	char *path_c;

	path_c = fstrnwcp( path, len_path );
	iret = sbf_unlink( path_c );
	free(path_c);

	return( iret );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfnfiles - Set number of files that make up SBF dataset
/*     
/*  SYNOPSIS:
/*    integer nfiles
/*    ier = sbfnfiles( nfiles )
/*
/*  DESCRIPTION:
/*    Sets the number of files that will be used for software
/*    striping of TRC data.
/*
/*--------------------------------------------------------------------*/

int sbfnfiles_( nfiles )
int *nfiles;
{
	return( sbf_nfiles( *nfiles ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfqfiles - Return number of files in SBF dataset
/*     
/*  SYNOPSIS:
/*    integer handle
/*    integer nfiles
/*    nfiles = sbfqfiles( handle )
/*
/*  DESCRIPTION:
/*    Returns number of .TRC files in the SBF dataset, -1 on errors.
/*
/*--------------------------------------------------------------------*/

int sbfqfiles_( handle )
int *handle;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Check for valid structure */
	if (sbf->magic != SBF_MAGIC) {
		sbf_error = SBF_BAD;
		return( -1 );
	}

/* Return number of files */
	return( sbf->nfiles );

}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfpaths - Set leading paths for multi-file SBF datasets
/*     
/*  SYNOPSIS:
/*    character*(*) pathfile
/*    integer ier
/*    ier = sbfpaths( pathfile )
/*
/*  DESCRIPTION:
/*    Sets leading path component for subsequent 'sbfcreat' calls.
/*    See comments for 'sbf_paths' in 'sbfio.c'.
/*
/*--------------------------------------------------------------------*/

int sbfpaths_( pathfile, len_p )
char *pathfile;
int len_p;
{
	char *path_c;
	int iret;

	path_c = fstrnwcp( pathfile, len_p );
	iret = sbf_paths( path_c );
	free(path_c);
	return( iret );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfcoord - Set coordinate type for subsequent sbf_creat calls
/*     
/*  SYNOPSIS:
/*    integer ctype
/*    call sbfcoord( ctype )
/*
/*  DESCRIPTION:
/*    Sets the coordinate type for subsequent sbf_creat calls.
/*    Set to 0 for uniform (no .XYZ file), 1 for ShotRcvr.
/*
/*--------------------------------------------------------------------*/

void sbfcoord_( ctype )
int *ctype;
{
	sbf_coord( *ctype );
	return;
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfctype - Return coord type of an open SBF dataset
/*     
/*  SYNOPSIS:
/*    integer handle
/*    integer ctype
/*    integer sbfctype
/*    ctype = sbfctype( handle )
/*
/*  DESCRIPTION:
/*    Returns coordinate type for an open SBF dataset, -1 on errors.
/*
/*--------------------------------------------------------------------*/

int sbfctype_( handle )
int *handle;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Check for valid structure */
	if (sbf->magic != SBF_MAGIC) {
		sbf_error = SBF_BAD;
		return( -1 );
	}

/* Return coordinate type */
	return( sbf->ctype );

}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfread - Read traces and coords from an SBF dataset
/*     
/*  SYNOPSIS:
/*    integer handle, frame, volume, ldim, maxtrc
/*    real xyz, trc
/*    ier = sbfread( handle, frame, volume, ldim, maxtrc, trc, xyz )
/*
/*  DESCRIPTION:
/*    Read traces and coordinates for a given frame and volume
/*    in an open SBF dataset.
/*
/*--------------------------------------------------------------------*/

int sbfread_( handle, frame, volume, ldim, maxtrc, trc, xyz )
int *handle, *frame, *volume, *ldim, *maxtrc;
float *trc, *xyz;
{
	SBFIO *sbf;
	int ier;

/* De-reference handle and set volume */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);
	if (sbf_setvol(sbf, *volume-1)) return(-1);

/* Read coords */
	ier = 0;
	if (sbf->ctype == ShotRcvr)
		ier = sbf_rdcoord( sbf, *frame-1, xyz );

/* Read traces */
	if (sbf_rdframe( sbf, *frame-1, trc )) return(-1);

/* Expand trace array if needed */
	if (*ldim > sbf->ns || *maxtrc > sbf->ntpf)
		sbf_copy( *ldim, *maxtrc, trc, sbf->ns, sbf->ntpf, trc );

	return(ier);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfwrite - Write traces and coords to an SBF dataset
/*     
/*  SYNOPSIS:
/*    integer handle, frame, volume, ldim, maxtrc
/*    real xyz, trc
/*    ier = sbfwrite(handle, frame, volume, ldim, maxtrc, trc, xyz)
/*
/*  DESCRIPTION:
/*    Writes traces and coordinates for a given frame and volume
/*    to an open SBF dataset.
/*
/*--------------------------------------------------------------------*/

int sbfwrite_( handle, frame, volume, ldim, maxtrc, trc, xyz )
int *handle, *frame, *volume, *ldim, *maxtrc;
float *trc, *xyz;
{
	SBFIO *sbf;
	int ier;

/* De-reference handle and set volume position */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);
	if (sbf_setvol(sbf, *volume-1)) return(-1);

/* Write coords */
	if (sbf->ctype == ShotRcvr)
		if (sbf_wrcoord( sbf, *frame-1, xyz )) return(-1);

	if (*ldim > sbf->ns) {
/* Compress trace array into temp area if needed */
		sbf_copy( sbf->ns, sbf->ntpf, trc, *ldim, *maxtrc, trc );
		ier = sbf_wrframe( sbf, *frame-1, trc ); 
		if (ier) return(-1);
		sbf_copy( *ldim, *maxtrc, trc, sbf->ns, sbf->ntpf, trc );
	}
	else
/* Write traces directly from input buffer */
		if (sbf_wrframe( sbf, *frame-1, trc )) return(-1);

	return(0);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfreadt - Read transposed traces and coords from an SBF dataset
/*     
/*  SYNOPSIS:
/*    integer handle, frame, volume, ldim, maxtrc
/*    real xyz, trc
/*    ier = sbfreadt( handle, frame, volume, ldim, maxtrc, trc, xyz )
/*
/*  DESCRIPTION:
/*    Read transposed traces and coordinates for a given frame and
/*    volume in an open SBF dataset.
/*
/*--------------------------------------------------------------------*/

int sbfreadt_( handle, frame, volume, ldim, maxtrc, trc, xyz )
int *handle, *frame, *volume, *ldim, *maxtrc;
float *trc, *xyz;
{
	SBFIO *sbf;
	int ier;

/* De-reference handle and set volume */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);
	if (sbf_setvol(sbf, *volume-1)) return(-1);

/* Read coords */
	ier = 0;
	if (sbf->ctype == ShotRcvr)
		if (sbf_rdcoordt( sbf, *frame-1, xyz )) ier = -2;

/* Read traces */
	if (sbf_rdframet( sbf, *frame-1, trc )) return(-1);

/* Expand trace array if needed */
	if (*ldim > sbf->ns || *maxtrc > sbf->nfpv ) 
		sbf_copy( *ldim, *maxtrc, trc, sbf->ns, sbf->nfpv, trc );

	return(ier);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfwritet - Write transposed traces and coords to an SBF dataset
/*     
/*  SYNOPSIS:
/*    integer handle, frame, volume, ldim, maxtrc
/*    real xyz, trc
/*    ier = sbfwrite( handle, frame, volume, ldim, maxtrc, trc, xyz )
/*
/*  DESCRIPTION:
/*    Writes transposed traces and coordinates for a given frame and 
/*    volume to an open SBF dataset.
/*
/*--------------------------------------------------------------------*/

int sbfwritet_( handle, frame, volume, ldim, maxtrc, trc, xyz )
int *handle, *frame, *volume, *ldim, *maxtrc;
float *trc, *xyz;
{
	SBFIO *sbf;
	int ier;

/* De-reference handle and set volume position */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);
	if (sbf_setvol(sbf, *volume-1)) return(-1);

/* Write coords */
	if (sbf->ctype == ShotRcvr) 
		if (sbf_wrcoordt( sbf, *frame-1, xyz )) return(-1);

	if (*ldim > sbf->ns) {
/* Compress trace array down to frame size if needed */
		sbf_copy( sbf->ns, sbf->ntpf, trc, *ldim, sbf->ntpf, trc );
		ier = sbf_wrframet( sbf, *frame-1, trc ); 
		if (ier) return(-1);
		sbf_copy( *ldim, sbf->ntpf, trc, sbf->ns, sbf->ntpf, trc );
	}
	else
/* Write traces directly from input buffer */
		if (sbf_wrframet( sbf, *frame-1, trc )) return(-1);

	return(0);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfreadp - Read traces and coords in panel mode
/*     
/*  SYNOPSIS:
/*    integer handle, frame, volume, ldim, maxtrc
/*    real xyz, trc
/*    ier = sbfreadp( handle, frame, volume, ldim, maxtrc, trc, xyz )
/*
/*  DESCRIPTION:
/*    Reads traces and coordinates in paneled mode.  In panel mode,
/*    a single frame is spread across the nodes.
/*
/*--------------------------------------------------------------------*/

int sbfreadp_( handle, frame, volume, ldim, maxtrc, trc, xyz )
int *handle, *frame, *volume, *ldim, *maxtrc;
float *trc, *xyz;
{
	SBFIO *sbf;
	int ntpn, nlive, ier;

/* De-reference handle and set volume */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);
	if (sbf_setvol(sbf, *volume-1)) return(-1);

/* Read coords */
	ier = 0;
	if (sbf->ctype == ShotRcvr)
		ier = sbf_rdcoordp( sbf, *frame-1, xyz );

/* Read traces */
	if (sbf_rdframep( sbf, *frame-1, trc )) return(-1);

/* Expand trace array if needed */
	jdcomp( sbf->ntpf, &ntpn, &nlive );
	if (*ldim > sbf->ns || *maxtrc > nlive ) 
		sbf_copy( *ldim, *maxtrc, trc, sbf->ns, nlive, trc );

	return(ier);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfwritep - Write traces and coords in panel mode
/*     
/*  SYNOPSIS:
/*    integer handle, frame, volume, ldim, maxtrc
/*    real xyz, trc
/*    ier = sbfwritep( handle, frame, volume, ldim, maxtrc, trc, xyz )
/*
/*  DESCRIPTION:
/*    Writes traces and coordinates in paneled mode.  In panel mode,
/*    a single frame is spread across the nodes.
/*
/*--------------------------------------------------------------------*/

int sbfwritep_( handle, frame, volume, ldim, maxtrc, trc, xyz )
int *handle, *frame, *volume, *ldim, *maxtrc;
float *trc, *xyz;
{
	SBFIO *sbf;
	int ier, ntpn, nlive;

/* De-reference handle and set volume position */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);
	if (sbf_setvol(sbf, *volume-1)) return(-1);

/* Write coords */
	if (sbf->ctype == ShotRcvr) 
		if (sbf_wrcoordp( sbf, *frame-1, xyz )) return(-1);

	if (*ldim > sbf->ns) {
/* Compress trace array into temp area if needed */
		jdcomp( sbf->ntpf, &ntpn, &nlive );
		sbf_copy( sbf->ns, nlive, trc, *ldim, nlive, trc );
		ier = sbf_wrframep( sbf, *frame-1, trc ); 
		if (ier) return(-1);
		sbf_copy( *ldim, nlive, trc, sbf->ns, nlive, trc );
	}
	else
/* Write traces directly from input buffer */
		if (sbf_wrframep( sbf, *frame-1, trc )) return(-1);

	return(0);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfrdfrm, sbfrdfrmt, sbfrdfrmp - Read traces from SBF .TRC file
/*     
/*  SYNOPSIS:
/*    integer handle, frame
/*    real x(*)
/*    ier = sbfrdfrm( handle, frame, buf )
/*
/*  DESCRIPTION:
/*    Read traces only from an open SBF dataset.
/*    Frame number is indexed as in fortran, i.e. first frame is 1.
/*
/*--------------------------------------------------------------------*/

int sbfrdfrm_( handle, frame, buf )
int *handle, *frame;
float *buf;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Read traces */
	return( sbf_rdframe( sbf, *frame-1, buf ) );
}

int sbfrdfrmt_( handle, frame, buf )
int *handle, *frame;
float *buf;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Read traces */
	return( sbf_rdframet( sbf, *frame-1, buf ) );
}

int sbfrdfrmp_( handle, frame, buf )
int *handle, *frame;
float *buf;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Read traces */
	return( sbf_rdframep( sbf, *frame-1, buf ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfwrfrm, sbfwrfrmt, sbfwrfrmp - Write traces to SBF .TRC file
/*     
/*  SYNOPSIS:
/*    integer handle, frame
/*    real x(*)
/*    ier = sbfwrfrm( handle, frame, buf )
/*
/*  DESCRIPTION:
/*    Write traces only to an open SBF dataset.
/*
/*--------------------------------------------------------------------*/

int sbfwrfrm_( handle, frame, buf )
int *handle, *frame;
float *buf;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Write traces */
	return( sbf_wrframe( sbf, *frame-1, buf ) );
}

int sbfwrfrmt_( handle, frame, buf )
int *handle, *frame;
float *buf;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Write traces */
	return( sbf_wrframet( sbf, *frame-1, buf ) );
}

int sbfwrfrmp_( handle, frame, buf )
int *handle, *frame;
float *buf;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Write traces */
	return( sbf_wrframep( sbf, *frame-1, buf ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfrdprm - Read parameter from SBF .HDR file
/*     
/*  SYNOPSIS:
/*    integer handle
/*    character*(*) name
/*    character*1 type
/*    char | real | integer value
/*    ier = sbfrdprm( handle, name, type, value )
/*
/*  DESCRIPTION:
/*    Reads a 'name=value' record from the .HDR file of an open
/*    SBF dataset.  'type' indicates the whether the expected 
/*    'value' is character (type='C'), float (type='F'), or 
/*    integer (type='I').
/*
/*--------------------------------------------------------------------*/

int sbfrdprm_( handle, name, type, buf, len_name, len_type, len_buf )
int *handle;
char *name;
char *type;
char *buf;
int len_name;
int len_type;
int len_buf;
{
	SBFIO *sbf;
	char *name_c;
	int iret;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Allocate space for null-terminated 'C' string */
	name_c = fstrnwcp(name,len_name);

/* Make call, then free temp space */
	iret = sbf_rdparm(sbf, name_c, *type, buf);
	free(name_c);
	return(iret);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfwrprm - Write parameter to SBF .HDR file
/*     
/*  SYNOPSIS:
/*    integer handle
/*    character*(*) name
/*    character*1 type
/*    char | real | integer value
/*    ier = sbfwrprm( handle, name, type, value )
/*
/*  DESCRIPTION:
/*    Writes a 'name=value' record to the .HDR file of an open
/*    SBF dataset.  'type' indicates the whether the input 
/*    'value' is character (type='C'), float (type='F'), or 
/*    integer (type='I').
/*
/*--------------------------------------------------------------------*/

int sbfwrprm_( handle, name, type, buf, len_name, len_type, len_buf )
int *handle;
char *name;
char *type;
char *buf;
int len_name;
int len_type;
int len_buf;
{
	SBFIO *sbf;
	char *name_c, *buf_c;
	int iret;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Allocate space for null-terminated 'C' string */
	name_c = fstrnwcp(name,len_name);

/* Make call based on type */
	if (*type == 'C') {
		buf_c = fstrncpy(buf,len_buf);
		iret = sbf_wrparm( sbf, name_c, *type, buf_c );
		free(buf_c);
	}
	else
		iret = sbf_wrparm( sbf, name_c, *type, buf );

	free(name_c);
	return( iret );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfqatr - Query attribute existence in SBF .ATR file
/*     
/*  SYNOPSIS:
/*    integer handle
/*    character*(*) name
/*    integer len
/*    len = sbfqatr( handle, name )
/*
/*  DESCRIPTION:
/*    Returns length in bytes of attribute in .ATR file if it exists,
/*    -1 on errors.
/*
/*--------------------------------------------------------------------*/

int sbfqatr_( handle, name, len_name )
int *handle;
char *name;
int len_name;
{
	SBFIO *sbf;
	char *name_c;
	int iret;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Allocate space for null-terminated 'C' string */
	name_c = fstrnwcp(name,len_name);

/* Make call, then free temp space */
	iret = sbf_qattr(sbf, name_c);
	free(name_c);
	return(iret);
}


/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfrdatr - Read attribute from SBF .ATR file
/*     
/*  SYNOPSIS:
/*    integer handle
/*    character*(*) name
/*    integer buf(*)
/*    ier = sbfrdatr( handle, name, buf )
/*
/*  DESCRIPTION:
/*    Reads a named attribute from the .ATR file of an open SBF
/*    dataset.  Returns length in bytes of attribute array on success,
/*    -1 on errors.
/*
/*--------------------------------------------------------------------*/

int sbfrdatr_( handle, name,  buf, len_name )
int *handle;
char *name;
int *buf;
int len_name;
{
	SBFIO *sbf;
	char *name_c;
	int iret;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Allocate space for null-terminated 'C' string */
	name_c = fstrnwcp(name,len_name);

/* Make call, then free temp space */
	iret = sbf_rdattr(sbf, name_c, buf);
	free(name_c);
	return(iret);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfwratr - Write attribute array to SBF .ATR file
/*     
/*  SYNOPSIS:
/*    integer handle
/*    character*(*) name
/*    integer buf(*)
/*    ier = sbfwratr( handle, name, buf, len )
/*
/*  DESCRIPTION:
/*    Writes a named attribute array to the .ATR file of an open SBF
/*    dataset.  Returns zero on success, -1 on errors.
/*
/*--------------------------------------------------------------------*/

int sbfwratr_( handle, name,  buf, len, len_name )
int *handle;
char *name;
int *buf;
int *len;
int len_name;
{
	SBFIO *sbf;
	char *name_c;
	int iret;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Allocate space for null-terminated 'C' string */
	name_c = fstrnwcp(name,len_name);

/* Make call, then free temp space */
	iret = sbf_wrattr(sbf, name_c, buf, *len);
	free(name_c);
	return(iret);
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfinfo - Return basic info from SBF I/O structure
/*     
/*  SYNOPSIS:
/*    integer handle
/*    integer ns, ntpf, nfpv, nvol
/*    ier = sbfinfo( handle, ns, ntpf, nfpv, nvol )
/*
/*  DESCRIPTION:
/*    Returns size variables for an open SBF dataset.
/*    'ns' is samples per traces, 'ntpf' is traces per frame,
/*    'nfpv' is frames per volume, and 'nvol' is number of volumes.
/*
/*--------------------------------------------------------------------*/

int sbfinfo_( handle, ns, ntpf, nfpv, nvol )
int *handle, *ns, *ntpf, *nfpv, *nvol;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Check for valid structure */
	if (sbf->magic != SBF_MAGIC) {
		sbf_error = SBF_BAD;
		return( -1 );
	}

/* Set file info */
	*ns = sbf->ns;
	*ntpf = sbf->ntpf;
	*nfpv = sbf->nfpv;
	*nvol = sbf->nvol;

	return( 0 );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfrinfo - Read aux parameters from SBF .HDR file
/*     
/*  SYNOPSIS:
/*    integer handle, msr
/*    real x0, y0, dx, dy, rot
/*    character*8 gtype  
/*    ier = sbfrinfo( handle, msr, x0, y0, dx, dy, rot, gtype )
/*
/*  DESCRIPTION:
/*    Reads auxilliary paramters from the .HDR file of an open
/*    SBF dataset. 'msr' is the sample rate in msec, (x0,y0) is the 
/*    orgin of the the CMP grid with spacing (dx,dy) and rotation
/*    'rot' in degrees.  'type' specifies the type of data stored.
/*
/*--------------------------------------------------------------------*/

int sbfrinfo_( handle, msr, x0, y0, dx, dy, rot, type, len_type )
int *handle, *msr;
float *x0, *y0, *dx, *dy, *rot; 
char *type;
int len_type;
{
	SBFIO *sbf;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Check for valid structure */
	if (sbf->magic != SBF_MAGIC) {
		sbf_error = SBF_BAD;
		return( -1 );
	}

/* Read info from file */
	sbf_rdparm( sbf, "SampleRate", 'I', msr );
	sbf_rdparm( sbf, "GridX0", 'F', x0 );
	sbf_rdparm( sbf, "GridY0", 'F', y0 );
	sbf_rdparm( sbf, "GridDX", 'F', dx );
	sbf_rdparm( sbf, "GridDY", 'F', dy );
	sbf_rdparm( sbf, "GridRot", 'F', rot );
	sbf_rdparm( sbf, "FrameType", 'C', type );

	return( 0 );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfwinfo - Write aux parameters to SBF .HDR file
/*     
/*  SYNOPSIS:
/*    integer handle, msr
/*    real x0, y0, dx, dy, rot
/*    character*8 type  
/*    ier = sbfwinfo( handle, msr, x0, y0, dx, dy, rot, type )
/*
/*  DESCRIPTION:
/*    Writes auxilliary parameters to the .HDR file of an open
/*    SBF dataset. 'msr' is the sample rate in msec, (x0,y0) is the 
/*    orgin of the the CMP grid with spacing (dx,dy) and rotation
/*    'rot' in degrees.  'type' specifies the type of data stored.
/*
/*--------------------------------------------------------------------*/

int sbfwinfo_( handle, msr, x0, y0, dx, dy, rot, 
	type, len_type )
int *handle, *msr;
float *x0, *y0, *dx, *dy, *rot; 
char *type;
int len_type;
{
	SBFIO *sbf;
	char *type_c;

/* De-reference handle */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

/* Allocate space for null-terminated 'C' string */
	type_c = fstrnwcp(type,len_type);

/* Write info to file */
	sbf_wrparm( sbf, "SampleRate", 'I', msr );
	sbf_wrparm( sbf, "GridX0", 'F', x0 );
	sbf_wrparm( sbf, "GridY0", 'F', y0 );
	sbf_wrparm( sbf, "GridDX", 'F', dx );
	sbf_wrparm( sbf, "GridDY", 'F', dy );
	sbf_wrparm( sbf, "GridRot", 'F', rot );
	sbf_wrparm( sbf, "FrameType", 'C', type_c );

/* Free temp space and return */
	free(type_c);
	return( 0 );
}

/*--------------------------------------------------------------------*/
/*  Write error explanation to standard error (sbf_perror)
/*--------------------------------------------------------------------*/

int sbferror_()
{
	return( sbf_perror(stderr,"sbfio:") );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    sbfsetvol - Set volume number in SBF dataset 
/*     
/*  SYNOPSIS:
/*    integer handle, ivol, ier, sbfsetvol
/*    ier = sbfsetvol( handle, ivol  )
/*
/*  DESCRIPTION:
/*    Sets volume number for subsequent reads/writes to a SBF dataset.
/*    Volume number has Fortran range (1:VolumesPerFile)
/*    Returns zero on success, negative on errors.
/*
/*--------------------------------------------------------------------*/

int sbfsetvol_( handle, ivol )
int *handle, *ivol;
{
	SBFIO *sbf;

/* Dereference pointer and set volume */
	sbf = sbf_handle(*handle);
	if (sbf == (SBFIO *)NULL) return(-1);

	return( sbf_setvol( sbf, *ivol-1 ) );
}
