/*--------------------------------------------------------------------*/
/*  NAME:
/*    message.c - message passing routines for ARCO/Perfect benchmark
/*
/*  VERSION:
/*    Intel iPSC/860 NX message passing
/*
/*  Routines defined in this module:
/*    jpinit - parallel initialization
/*    jmaster - return master node number
/*    jnodes - return number of nodes in system
/*    jdim - return hypercube dimension
/*    jnode - return node number
/*    jsync - global sync
/*    jkill - kill all processes
/*    jsend - send message (synchronous)
/*    jrecv - receive message (synchronous)
/*    jasend - send message (async)
/*    jarecv - receive message (async)
/*    jamsgwt - async message wait
/*    jbrdcst - broadcast message 
/*    jtrace - toggle tracing
/*    jdcomp - set simple decomposition
/*    jrange - return index range for simple decomposition
/*    jtran23 - tiled transpose for simple decomposition
/*     
/*--------------------------------------------------------------------*/
/*
/*  HISTORY:
/*
/*     Written:
/*       01/19/92 - C. Mosher, ARCO E&PT 
/*
/*     References:
/*       Intel iPSC/860 System Documentation
/*
/*     Modified:
/*
/*--------------------------------------------------------------------*/

/*--------------------------------------------------------------------*/
/*  Global variables
/*--------------------------------------------------------------------*/

/* Trace flag */
static int trace = 0;

/* Node numbers, master node, task id array */
static int node, nodes, master, *tids;

/* Data decomposition parameters */
static int nglob=0, ntpn=0, nlive=0;

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jpinit - Initialize parallel processing
/*     
/*  SYNOPSIS:
/*
/*    char *path;
/*    int iret;
/*    iret = jpinit( path );
/*
/*  Initiates parallel processing environment.  For machines that
/*  require host or node initialization, the 'path' variable can
/*  be used to specify the executable to be loaded.
/*
/*  PVM Notes:
/*  Assumes standard PVM installation, and that 'pvmd' has been
/*  started.  Use the 'pvm_config' routine to find out how many
/*  machines are in the cluster.  The number of nodes started is
/*  set to the number of machines in the cluster.  If evironment
/*  variable 'PVM_NODES' is set, this number is used instead.
/*
/*--------------------------------------------------------------------*/

int jpinit( path )
char *path;
{

	nodes = 1;
	node = 0;
	master = 0;

/* Set node numbers using NX calls */
	node = mynode();
	nodes = numnodes();
	master = 0;

	return( 0 );
}

/* Fortran interface */
int jpinit_( path )
char *path;
{
	return( jpinit( path ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jpterm - terminate parallel processing
/*     
/*  SYNOPSIS:
/*
/*    ier = jpterm();
/*
/*--------------------------------------------------------------------*/

int jpterm()
{
	return( 0 );
}

int jpterm_()
{
	return( jpterm() );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jmaster - return master node number
/*     
/*  SYNOPSIS:
/*
/*    integer master
/*    master = jmaster( )
/*
/*--------------------------------------------------------------------*/

int jmaster()
{
	if (trace)
		printf("jmaster node %d\n",node);
	return(0);
}

int jmaster_()
{
	return( jmaster() );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jnodes - return number of nodes in message passing complex
/*     
/*  SYNOPSIS:
/*
/*    integer nodes
/*    nodes = jnodes( )
/*
/*--------------------------------------------------------------------*/

int jnodes()
{
	if (trace)
		printf("jnodes  node %d\n",node);
	return( nodes );
}

int jnodes_()
{
	return( nodes );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jnode - return processor node number
/*     
/*  SYNOPSIS:
/*
/*    integer node
/*    node = jnode( )
/*
/*--------------------------------------------------------------------*/

int jnode()
{
	if (trace)
		printf("jnode   node %d\n",node);
	return( node );
}

int jnode_()
{
	return( node );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jkill - issue kill signal to entire processor complex
/*     
/*  SYNOPSIS:
/*
/*    call jkill( )
/*
/*--------------------------------------------------------------------*/

int jkill()
{
	int i,iret;

	iret = 0;
	if (trace)
		printf("jkill   node %d\n",node);

	iret = killcube( -1, 0 );

	return( iret );
}

int jkill_()
{
	return( jkill() );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jsync - issue global sync signal to entire processor complex
/*     
/*  SYNOPSIS:
/*
/*    call jsync( )
/*
/*--------------------------------------------------------------------*/

int jsync()
{

	if (trace)
		printf("jsync   node %d\n",node);

/* iPSC global sync routine */
	return( gsync() );

	return( 0 );
}

int jsync_()
{
	return( jsync() );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jsend - send message
/*     
/*  SYNOPSIS:
/*
/*    integer type, length, dest
/*    character*1 buf(length)
/*    call jsend( type, buf, length, dest )
/*
/*--------------------------------------------------------------------*/

int jsend( type, buf, length, dest )
int type, length, dest;
char *buf;
{
	int i,iret;

	iret = 0;
	if (trace)
		printf("jsend   node %d: type, buf, length, dest = %d 0x%x %d %d\n",
			node,type,buf,length,dest);

	iret = csend( type, buf, length, dest, 0 );

	return( iret );
}

int jsend_( type, buf, length, node )
int *type, *length, *node;
char *buf;
{
	return( jsend( *type, buf, *length, *node ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jrecv - receive next message
/*     
/*  SYNOPSIS:
/*
/*    integer type, length
/*    character*1 buf(length)
/*    call jrecv( type, buf, length )
/*
/*--------------------------------------------------------------------*/

int jrecv( type, buf, length )
int type, length;
char *buf;
{
	int len,iret;

	iret = 0;
	if (trace)
		printf("jrecv   node %d: type, buf, length = %d 0x%x %d\n",
			node,type,buf,length);

	iret = crecv( type, buf, length );

	return( iret );
}

int jrecv_( type, buf, length )
int *type, *length;
char *buf;
{
	return( jrecv( *type, buf, *length ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jasend - asynchronous send message
/*     
/*  SYNOPSIS:
/*
/*    integer msg, type, length, dest
/*    character*1 buf(length)
/*    msg = jasend( type, buf, length, dest )
/*
/*--------------------------------------------------------------------*/

int jasend( type, buf, length, dest )
int type, length, dest;
char *buf;
{
	int iret;

	iret = 0;
	if (trace)
		printf("jasend   node %d: type, buf, length, dest = %d 0x%x %d %d\n",
			node,type,buf,length,dest);

	iret = isend( type, buf, length, dest, 0 );

	return( iret );
}

int jasend_( type, buf, length, node )
int *type, *length, *node;
char *buf;
{
	return( jasend( *type, buf, *length, *node ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jarecv - asynchronous message receive
/*     
/*  SYNOPSIS:
/*
/*    integer msg, type, node, length
/*    character*1 buf(length)
/*    msg = jarecv( type, buf, length, node )
/*
/*--------------------------------------------------------------------*/

int jarecv( type, buf, length )
int type, length;
char *buf;
{
	int iret;

	iret = 0;
	if (trace)
		printf("jarecv   node %d: type, buf, length = %d 0x%x %d\n",
			node,type,buf,length);

	iret = irecv( type, buf, length );

	return( iret );
}

int jarecv_( type, buf, length )
int *type, *length;
char *buf;
{
	return( jarecv( *type, buf, *length ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jamsgwt - asynchronous message receive
/*     
/*  SYNOPSIS:
/*
/*    integer id
/*    call jamsgwt( id )
/*
/*--------------------------------------------------------------------*/

int jamsgwt( id )
int id;
{
	int iret;

	iret = 0;
	if (trace)
		printf("jamsgwt   node %d: id = %d\n", node, id);

	msgwait( id );

	return( iret );
}

int jamsgwt_( id )
int *id;
{
	return( jamsgwt( *id ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jbrdcst - broadcast message to all nodes
/*     
/*  SYNOPSIS:
/*
/*    integer type, length, sender
/*    character*1 buf(length)
/*    call jbrdcst( type, buf, length, sender )
/*
/*--------------------------------------------------------------------*/

int jbrdcst( type, buf, length, sender )
int type, length, sender;
char *buf;
{
	int iret;

	iret = 0;
	if (trace)
		printf("jbrdcst node %d: type, buf, length, sender = %d 0x%x %d %d\n",
			node,type,buf,length,sender);

	if (node == sender)
		iret = csend( type, buf, length, -1, 0 );
	else
		iret = crecv( type, buf, length );

	return( iret );
}

jbrdcst_( type, buf, length, sender )
int *type, *length, *sender;
char *buf;
{
	return( jbrdcst( *type, buf, *length, *sender ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jtrace - Toggle trace option
/*     
/*  SYNOPSIS:
/*
/*    integer iop
/*    call jtrace( iop )
/*
/*--------------------------------------------------------------------*/

int jtrace( iop )
int iop;
{
	trace = iop;
	if (trace)
		printf("jtrace  node %d: tracing is on, nodes, master = %d %d\n",node,nodes,master);
	else
		printf("jtrace  node %d: tracing is off\n",node);

	return(0);
}

int jtrace_( iop )
int *iop;
{
	return( jtrace( *iop ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jdcomp - Simple domain decomposition
/*    Allocates a block of data for each processor
/*     
/*  SYNOPSIS:
/*
/*    integer n, jntpn, jnlive
/*    call jdcomp( n, jntpn, jnlive )
/*
/*  DESCRIPTION:
/*    Divides 'n' by number of processors, and allocates 'jntpn'
/*    values to nodes 0 to n-2, and 'jnlive' values to node n-1.
/*    If n is an even multiple of the number of processors, then
/*    jnlive = jntpn.
/*
/*--------------------------------------------------------------------*/

int jdcomp( n, jntpn, jnlive )
int n, *jntpn, *jnlive;
{
	int nrem, nfull;

/* Compute number of traces to place on last processor */
	nrem = n % nodes;
	if (nrem > 0) {
		ntpn = (n/nodes) + 1;
		nfull = n/ntpn;
		if (node >= nfull) {
			nlive = n - (ntpn*node);
			if (nlive < 0) nlive = 0;
		}
		else
			nlive = ntpn;
	}
	else {
		ntpn = n/nodes;
		nlive = ntpn;
	}

	nglob = n;
	*jntpn = ntpn;
	*jnlive = nlive;

	if (trace)
		printf("jdcomp  node %d: nglob, ntpn, nlive = %d %d %d\n",
			node,nglob,ntpn,nlive );

	return(0);
}

/* Fortran interface */

int jdcomp_( n, jntpn, jnlive )
int *n, *jntpn, *jnlive;
{
	return( jdcomp( *n, jntpn, jnlive ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jrange - simple index range computation
/*     
/*  SYNOPSIS:
/*    integer n, j1, jn
/*    call jrange( n, j1, jn )
/*
/*  DESCRIPTION:
/*    Allocates 'n' values across processors using 'jdcomp', then
/*    returns the global index range of the values
/*    allocated to the current processor.  If no values are on this
/*    processor, returns -1 and a range of j1 = jn = 0.
/*
/*--------------------------------------------------------------------*/

int jrange( n, j1, jn )
int n, *j1, *jn;
{
	int iret;

/* Set decomposition if it's not done yet */
	if (n != nglob) jdcomp( n, &ntpn, &nlive );

/* Set range on this processor */
	if (nlive > 0) {
		*j1 = ntpn*node + 1;
		*jn = *j1 + nlive - 1;
		iret = 0;
	}
	else {
		*j1 = *jn = 0;
		iret = -1;
	}

	if (trace)
		printf("jrange node %d: n,j1,jn = %d %d %d\n",node,n,*j1,*jn);

	return( iret );
}

/* Fortran interface */

int jrange_( n, j1, jn )
int *n, *j1, *jn;
{
	return( jrange( *n, j1, jn ) );
}

/*--------------------------------------------------------------------*/
/*  NAME:
/*    jtran23 - Distributed matrix transpose primitive
/*     
/*  SYNOPSIS:
/*    integer n, a, buf
/*    call jtran23( n, a, buf )
/*
/*  DESCRIPTION:
/*    Transposes last two dimensions of an (n,p,p) matrix, where 'p'
/*    is the number of processors, and the last dimension is 
/*    allocated across processors as in 'jdcomp'.  Uses a standard 
/*    tiling algorithm, developed by Intel SSD. See 'trans.f'
/*    for examples of how this primitive can be used to apply more
/*    general transpose operations.
/*
/*--------------------------------------------------------------------*/

int jtran23( n, a, buf )
int n, *a, *buf;
{
#	define MSG 10000
	static int one = 1;
	int myswap, i, len, pmax, msg1, msg2;
	int *ap;

	if (nodes <= 1) return(0);

/* Length in bytes of 1 row (tile) of array */
	len = n * sizeof(int);
 
/* Find power of 2 => number of nodes */
	pmax = 1;
	while (pmax < nodes)
		pmax = 2*pmax;

/* Loop and exchange tiles */
	for (i=1; i<=pmax; i++) {
/* Processor to swap with is XOR of processor number and i */
		myswap = node ^ i;
/* tile number to send is same as target processor number */
		ap = a + n*myswap;
/* Swap tiles */
		if (myswap < nodes) {
			msg1 = irecv( MSG+i, buf, len );
			msg2 = isend( MSG+i, ap, len, myswap, 0 );
			msgwait(msg1);
			msgwait(msg2);
			scopy_( &n, buf, &one, ap, &one );
		}
	}

	return(0);
}

/* Fortran interface */

int jtran23_( n, a, buf )
int *n, *a, *buf;
{
	return( jtran23( *n, a, buf ) );
}
