C*----------------------------------------------------------------------
C*
C*  NAME:
C*    dconp, dcona, dconb, dconc - Seismic Processing Benchmark 
C*        process DCON
C*     
C*  SYNOPSIS:
C*    integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
C*    real otr(ldim,maxtrc), ra(nra), sa(nra)
C*    logical abort
C*    call dconp( ldim, maxtrc, nra, nsa, abort, ipr )
C*    call dcona( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                abort, ipr )
C*    call dconb( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntri, ntro, abort, ipr )
C*    call dconc( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntro, abort, ipr )
C*
C*  DESCRIPTION: 
C*    Applies gapped deconvolution to seismic traces
C*
C*  PARAMETERS: 
C*    Record 1
C*    Field  Columns   Description
C*      1     11-15    Gap length
C*      2     16-20    Operator length in msec
C*      3     21-25    White noise stabilization factor, %
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*     Include files:
C*       ./coord.inc
C*       ./dcon.inc
C*
C*     Libraries:
C*       libbench.a
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        12/31/91 - C. Mosher, ARCO E&PT 
C*
C*      References:
C*
C*      Modified:
C*
C*----------------------------------------------------------------------

C*----------------------------------------------------------------------
C*   NAME:
C*     dconp
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   This process reads 'DCON' parameters, checks syntax, and sets
C*   memory requirements for the processing phase
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*      Subroutines:
C*        getparm - read parameter record
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/06/90 - C. Mosher
C*
C*      References:
C*        SPARC System Documentation
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine dconp( ldim, maxtrc, nra, nsa, abort, ipr )
C
C  Arguments:
C
C  Output leading dimension of trace array required for the PROC step
	integer LDIM
C  Output max number of traces per group to be output during PROC step
	integer MAXTRC
C  Number of words of reserved storage required in PROC step
	integer NRA
C  Number of words of working storage required in PROC step
	integer NSA
C  Abort flag, set to .TRUE. by this process if errors detected
	logical ABORT
C  Print unit number that this routine should use
	integer IPR

C
C  Common blocks:
C
C  parameters for process DCON
	include 'dcon.inc'
C  data structure definition
	include 'group.inc'

C
C  Local declarations:
C
C  parameter record buffer
	character*80 buf
C  parameter record index
	integer index

C
C  Start of executable statements
C

C  Get parameter record
	index = 1
	call getparm('DCON',index,buf,ier)
	if (ier.ne.0) then
		write (ipr,*) 'No parameter records for process DCON'
		abort = .true.
		return
	endif

C  Read parameters
	read (buf,'(10x,3f5.0)') gap, oplen, wnois
	if (gap.eq.0. .or. oplen.eq.0.) then
		write (ipr,*) 'DCON: invalid gap/operator length'
		abort = .true.
		return
	endif
	if (wnois.eq.0.) wnois = 0.1

	write (ipr,9010) gap, oplen, wnois
 9010 format (' Process DCON    : Gapped Deconvolution',/,
     +        ' Gap length      : ',f8.1,/,
     +        ' Operator length : ',f8.1,/,
     +        ' Whitening (%)   : ',f8.2)

C  Convert times to samples
	igap = 1 + nint(gap/float(msr))
	lenop = 1 + nint(oplen/float(msr))

C  Set memory requirements
	ldim = ns
	maxtrc = ntpg
	nra = 0

C  Work array pointers
	len = ns + igap + lenop
	kr = 1
	kc = kr + len
	kh = kc + len
	ka = kh + len
	nsa = ka + len

	return
	end

C*----------------------------------------------------------------------
C*   NAME:
C*     dcona, dconb, dconc
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   Processing phase routines for dconing Seismic Benchmark 
C*     Format files
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/01/92 - C. Mosher
C*
C*      References:
C*        Seismic Benchmark Users Guide
C*
C*      Modified:
C*
C*----------------------------------------------------------------------

C*----------------------------------------------------------------------
C*  The 'a' routine does nothing
C*----------------------------------------------------------------------

	subroutine dcona( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		abort, ipr )
	integer ldim, maxtrc, nra, nsa, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort

	return
	end

C*----------------------------------------------------------------------
C*  The 'b' routine applies decon
C*----------------------------------------------------------------------

	subroutine dconb( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntri, ntro, abort, ipr )
	integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort

	include 'dcon.inc'
	include 'group.inc'

	do j=1,ntri
C  Compute and apply gapped operator
		call dcongap( ns, otr(1,j), igap, lenop, wnois,
     +		sa(kr), sa(kc), sa(kh), sa(ka) ) 
	enddo

	ntro = ntri
	return
	end

C*----------------------------------------------------------------------
C*  'c' routine does nothing
C*----------------------------------------------------------------------
c
	subroutine dconc( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntro, abort, ipr )
	integer ldim, maxtrc, nra, nsa, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort

	call sysmsg('Process DCON completed')

	return
	end

C*----------------------------------------------------------------------

	subroutine dcongap( ns, otr, igap, lenop, wnois, r, c, h, a )
C
C  Apply gapped decon
C
C  ns - samples per trace
C  otr - trace array
C  igap - gap length in samples
C  lenop - operator length
C  wnois - white noise percentage
C  r,c,h,a - work arrays, length ns + igap + lenop
C
C---------------------------------------------------------------------

	dimension otr(ns),r(*),c(*),h(*),a(*)
	integer i, len

C  Autocorrelate input trace
	len = igap + lenop
	call dconaco( otr, r, len, ns )
C  Delay autocorrelation to use as correlation function
	do i=1,lenop
		c(i) = r(igap+i)
	enddo
C  Add white noise to first element of autocor 
	r(1) = r(1) * (1.0 + 0.01*wnois)
C  Compute prediction operator
	call dconwnr( lenop, r, c, h, a )
C  Delay operator and negate
	r(1) = 1.0
	do i=2,igap
		r(i) = 0.0
	enddo
	do i = igap+1, len
		r(i) = -h(i-igap) 
	enddo
C  Convolve with trace
	call dconcnv( ns, len, otr, r, a )
C  Move from work array to output
	do i=1,ns
		otr(i) = a(i)
	enddo
	return
	end

C*----------------------------------------------------------------------

	subroutine dconaco( a, c, len, n )
c
c  Autocorrelation
c
c  a - input time series to be autocorrelated
c  c - output autocorrelation
c  len - number of autocorrelation values 
c  n - time series length
c
	integer n,i,j,k,i1
	real a(n),c(n),sum

c  Intial start point for dot product
	i1 = 1

c  Loop and perform dot product for each lag
	do j = 1,len
		sum = 0.0
		k = 1
		do i = i1,n
			sum = sum + a(i) * a(k)
			k = k + 1
		enddo
		c(j) = sum
c  Increment start point
		i1 = i1 + 1
	enddo

	return
	end

C*----------------------------------------------------------------------

      subroutine dconwnr(lr,r,g,f,a)
C
C    MATRIX INVERSION ROUTINE MODIFIED FROM ROBINSON'S BOOK
C
C    PARAMETERS
C
C    LR   -  NUMBER OF FILTER COEFFICIENTS TO BE COMPUTED
C    R    -  AUTOCORRELATION MATRIX
C    G    -  CROSS CORRELATION MATRIX
C    F    -  FILTER COMPUTED BY OUSCMP
C    A    -  ARRAY CONTAINING SPIKING DECONVOLUTION FILTER.
C
      DIMENSION R(2),G(2),F(3),A(4)
      V=R(1)
      D=R(2)
      A(1)=1.0
      F(1)=G(1)/V
      Q=F(1)*R(2)
      IF(LR.EQ.1)RETURN
      DO 4 L=2,LR
      A(L)=-D/V
      IF(L.EQ.2)GO TO 2
      L1=(L-2)/2
      L2=L1+1
      IF(L2.LT.2)GO TO 5
      DO 1 J=2,L2
      HOLD=A(J)
      K=L-J+1
      A(J)=A(J)+A(L)*A(K)
    1 A(K)=A(K)+A(L)*HOLD
    5 IF(2*L1.EQ.L-2)GO TO 2
      A(L2+1)=A(L2+1)+A(L)*A(L2+1)
    2 V=V+A(L)*D
      F(L)=(G(L)-Q)/V
      L3=L-1
      DO 3 J=1,L3
      K=L-J+1
    3 F(J)=F(J)+F(L)*A(K)
      IF(L.EQ.LR)RETURN
      D=0.0
      Q=0.0
      DO 4 I=1,L
      K=L-I+2
      D=D+A(I)*R(K)
    4 Q=Q+F(I)*R(K)
      RETURN
      END

C*----------------------------------------------------------------------

	subroutine dconcnv( n, m, a, b, c )
c
c  Convolution
c  n - time series length
c  m - filter length
c  a - input time series, length n
c  b - input filter, length m
c  c - output convolution, length n
c
	integer n,m,i,j,k,k1
	real a(n),b(m),c(n),sum
	
C Loop and convolve filter with input array
	do j = 1,n
c  Set first filter and input time series indices
		if (j.lt.m) then
			k1 = j
			i = 1
		else
			k1 = m
			i = j-m+1
		endif
c  Loop and form reverse dot product
		sum = 0.0
		do k = k1,1,-1
			sum = sum + a(i) * b(k)
			i = i + 1
		enddo
		c(j) = sum
	enddo

	return
	end
