C*========================= START OF MODULE ============================
C*
C*  Frequency-Space Post-Stack Depth Migration
C*  Version 2.0: Implicit 60 degree operator
C*
C*========================= START OF SUBROUTINE ========================
C*     
C*   NAME: fxmgp  
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   PREP Step for SeisPerf Seismic Benchmarks
C*     F-X Depth Migration
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*      Subroutines:
C*        getparm - get paramter record
C*
C*----------------------------------------------------------------------
C*   DEFICIENCIES:
C*    
C*       
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        08/20/92 - C. Mosher, L. Morley
C*
C*      References:
C*        Claerbout, 1985, Imaging the Earth's Interior
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine fxmgp( ldim, maxtrc, nra, nsa, abort, ipr )
	implicit none
C
C*----------------------------------------------------------------------
C*  ARGUMENTS:
C*----------------------------------------------------------------------
C
C  Output leading dimension of trace array required for the PROC step
	integer LDIM
C  Output max number of traces per group to be output during PROC step
	integer MAXTRC
C  Number of words of reserved storage required in PROC step
	integer NRA
C  Number of words of working storage required in PROC step
	integer NSA
C  Abort flag, set to .TRUE. by this process if errors detected
	logical ABORT
C  Print unit number that this routine should use
	integer IPR

C-----------------------------------------------------------------------
C  Include files
C-----------------------------------------------------------------------
	include 'group.inc'
	include 'coord.inc'
	include 'para.inc'
	include 'fxmg.inc'
C
C*----------------------------------------------------------------------
C*  Local declarations
C*----------------------------------------------------------------------
C
	character*80 data
	integer index, ier, len, mz, mt
	real fmax, sf
C
C*----------------------------------------------------------------------
C*  Start of executable statements
C*----------------------------------------------------------------------
C
C
C  read first card
C    Debug flag, max time, max depth, boundary points, max frequency,
C      input trace separation, depth step size, x loc of first trace,
C      constant migration velocity
C
	index = 1
	call getparm('FXMG',index,data,ier)
	if (ier.ne.0) goto 999
	read (data,'(9x,i1,2i5,4f5.0,f10.0)') 
     +	idbug,mt,mz,fmax,dx,dz,x1,vmig

C  Default trace spacing to existing attributes
	if (dx.eq.0.) then
		x1 = 0.0
		dx = gdx
	endif
C  Set number of grid points to number of input traces
	nx = ntpg
C  Force to doubleword location
	if (mod(nx,2).ne.0) nx = nx + 1

C  Default maximum time
	if (mt.eq.0 .or. mt.ge.msr*ns) mt = msr*(ns-1)
	nt = 1 + mt/msr

C  Default max frequency
	fsr = 1.e-3 * float(msr)
	if (fmax.eq.0.0) fmax = 0.5/fsr

C  Fourier transform parameters
	call seiftm( nt, mag, n2 )
	sf = 1./( fsr * n2 )
	sw = 8.*atan(1.)*sf
C  Set number of frequencies to be processed
	nf = 1 + nint(fmax/sf)
C  But don't exceed nyquist frequency
	if (nf. gt. n2/2) nf = n2/2
	fmax = sf*(nf-1)

C  Depth steps
	z1 = 0.0
	if (dz.eq.0.) dz = dx
	if (mz.eq.0) then
		nz = nx
		mz = dz*(nz-1)
	else
		nz = 1 + nint( float(mz)/dz )
	endif

C
C  Print and check finite difference parameters
	write (ipr,9010) mt,mz,nz,dz,nx,dx,fmax,vmig
 9010 format (/,' Post Stack FX Domain Depth Migration',//,
     +          ' Time Length (ms) :',i8,/,
     +          ' Max depth        :',i8,/,
     +          ' Depth Steps      :',i8,/,
     +          ' Z step size      :',f8.2,/,
     +          ' X Grid points    :',i8,/,
     +          ' X step size      :',f8.2,/,
     +          ' Max Frequency    :',f8.1,/,
     +          ' Migration Vel    :',f8.1)
c
	if (idbug.ne.0) write (ipr,*) ' Debug print is on'
	if (vmig.le.0. .or. dx.le.0.) then
		write (ipr,*) 'FXMG Error: required parameter is zero'
		abort = .true.
		return
	endif
C
C  Data must be in panelled mode if more than 1 processor
	if (nodes.gt.1 .and. pmode.ne.1) then
		write (ipr,*) 'FXMG Error: must be in panelled mode'
		abort = .true.
		return
	endif
C
C  Set output data decomposition
	call jdcomp( nx, jntpn, jnlive )
C
C  Internal decomposition over frequency
	call jdcomp( nf, nfpn, nflive )
C
C  set trace array dimensions and amount of storage required
C
C  Output traces are in depth and decomposed over midpoint
	LDIM = nz
	MAXTRC = jntpn

C  No reserved storage used
	NRA = 0
C  Work space for finite difference grid
	len = 2*jntpn*nfpn*nodes
	NSA = len
C  Other work arrays
	ksa = NSA + 1
	len = max( 12*nx, n2+2, 2*nfpn*jntpn )
	NSA = NSA + len 
	write (ipr,9040) LDIM,MAXTRC,NSA
 9040 format (/,' Trace array size : (',i5,',',i5,')',/,
     +          ' Working  storage : ',i8)

C  Set output attributes
	call fxmg_attr()

	return

C  Error exit
 999  continue
	write (ipr,*) ' *** Parameter records missing for process FXMG'
	abort = .true.
	return
	end

C*========================= START OF MODULE ============================
C*
C*  NAME:
C*    fxmga,fxmgb,fxmgc - OSPARC Plane Wave Depth Migration
C*
C*  SYNOPSIS:
C*    integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
C*    real otr, ra, sa;
C*    logical abort;
C*    call fxmga( ldim, maxtrc, otr, nra, ra, nsa, sa, 
C*                abort, ipr )
C*    call fxmgb( ldim, maxtrc, otr, nra, ra, nsa, sa, 
C*                ntri, ntro, abort, ipr )
C*    call fxmgc( ldim, maxtrc, otr, nra, ra, nsa, sa,  
C*                ntro, abort, ipr )
C*  
C*  DESCRIPTION:
C*    Frequency-Space 2D post-stack migration
C*
C*----------------------------------------------------------------------
C*
C*  DEPENDENCIES:
C*
C*----------------------------------------------------------------------
C*
C*  DIAGNOSTICS:
C*
C*----------------------------------------------------------------------

	subroutine fxmga(ldim,maxtrc,otr,nra,ra,nsa,sa,abort,ipr)
C
C  Parallel initialization routine -  called for first trace on line
C
C  ldim - leading dimension of trace array
C  maxtrc - max number of traces per gather
C  otr(ldim,*) - input array of traces
C  nra - length of reserved storage array
C  ra - reserved storage
C  nsa - length of working storage array
C  sa - working storage
C  abort - abort code, .true. = abort
C  ipr - print unit number
C
	implicit none
C  Arguments
	integer ldim,maxtrc,nra,nsa,ipr
	real otr(ldim,maxtrc),ra(nra),sa(nsa)
	logical abort
C  Include files
	include 'group.inc'
	include 'para.inc'
	include 'fxmg.inc'
C  local declarations

C-----------------------------------------------------------------------
C  Start of executable statements
C-----------------------------------------------------------------------
C
C  set abort flag
	abort = .false.

	if (idbug.ne.0 .and. node.eq.master) then
		idbug = ipr
	else
		idbug = 0
	endif

C  Set output data decomposition
	call jdcomp( nx, jntpn, jnlive )

C  Internal decomposition over frequency
	call jdcomp( nf, nfpn, nflive )

C  Set attributes
	call fxmg_attr()

C  Initialize timers
	tcomp = 0.0
	tcomm = 0.0

	return
	end

C
C***********************************************************************
C
	subroutine fxmgb( ldim, maxtrc, otr, nra, ra, nsa, sa,
     +    ntri, ntro, abort, ipr )
	implicit none

C  Arguments
	integer ldim,maxtrc,nra,nsa,ntri,ntro,ipr
	real otr(ldim,maxtrc),ra(nra),sa(nsa)
	logical abort
C  Include files
	include 'group.inc'
	include 'para.inc'
	include 'fxmg.inc'
C  Local declarations
	integer jf1,jfn,ldq
	real t0, etimer

C-----------------------------------------------------------------------

	abort = .false.
	ntro = nx

C  Initial elapsed time
	t0 = etimer(0)

C  Fourier transform up to max frequency and store transposed
	call fxmg_fft( ldim, otr, ldq, sa, sa(ksa) )

C  Frequency range on this node
	call jrange( nf, jf1, jfn ) 

C  Migrate
	call fxmg_migr( nx, x1, dx, nz, z1, dz, jf1, jfn, sw,
     +		ldq, sa, ldim, otr, sa(ksa), nt, n2, idbug )

C  Set computation time to elapsed time minus communication time
	tcomp = etimer(0) - t0 - tcomm

C  Set output attributes
	call fxmg_attr()

	return
	end

C
C***********************************************************************
C
	subroutine fxmgc(ldim,maxtrc,otr,nra,ra,nsa,sa,ntro,abort,ipr)

	integer ldim,maxtrc,nra,nsa,ntro,ipr
	real otr(ldim,maxtrc),ra(nra),sa(nsa)
	logical abort

C-----------------------------------------------------------------------
C  Include files
C-----------------------------------------------------------------------
	include 'group.inc'
	include 'para.inc'
	include 'fxmg.inc'
C  Local declarations
	real flopsm

	ntro = 0
	if (node.ne.master) return
C  Megaflops from regression on Cray YMP HPM data
	flopsm = 2.2e-4 * float(nf) * float(nx) * float(nz)
	write (ipr,9010) tcomp, tcomm, 1.e6*flopsm,
     +       flopsm/tcomp, flopsm/(tcomp + tcomm)
 9010   format (/,' Process FXMG Completed',/,
     +          ' 2D Depth Migration Statistics:',/,
     +          ' Compute time    : ',f10.1,/,
     +          ' Communicate time: ',f10.1,/,
     +          ' Flt pt ops (est): ',g12.4,/,
     +          ' Raw Mflops      : ',f10.2,/,
     +          ' Effective Mflops: ',f10.2,/)

	return
	end
C
C***********************************************************************
C
	
C
C-----------------------------------------------------------------------
C
	subroutine fxmg_fft( ldim, otr, ldq, q0, sa )
C
C  Fourier transform input wavefield and store transposed
C  with frequencies distributed across processors
C
C  ldim - leading dimension of trace array
C  otr(ldim,*) - input traces
C  ldq - output leading dimension of wavefield array
C  complex q0(ldq,*) - output wavefield at z=0
C  complex sa(*) - work array for transforms, length n2 + 2
C
	implicit none
C  Arguments
	integer ldim, ldq
	real otr(ldim,*), sa(*)
	complex q0(*)
C  Include files
	include 'para.inc'
	include 'fxmg.inc'
C  Local variables
	integer ldqt, nzr, izr1
	real t0, etimer

C  Dimensions of wavefield array
	ldqt = nfpn * nodes
	ldq = jntpn * nodes

C  Fourier transform input data over time
	if (nlive.gt.0)
     +	call seimrcf( ldim, nlive, otr, ldqt, q0, n2, nt, nf, sa )

C  Zero fill any edge traces
	if (nlive.lt.ntpn) then
		nzr = 2*ldqt*(ntpn-nlive)
		izr1 = 2*ldqt*nlive + 1
		call vfill( 0.0, q0(izr1), 1, nzr )
	endif

C  Transpose so that frequencies are distributed over processors
	t0 = etimer(0)
	call dtran21c( ldqt, ldq, q0, sa )
	tcomm = tcomm + etimer(1)

	return
	end

C
C-----------------------------------------------------------------------
C
	subroutine fxmg_migr( nx, x1, dx, nz, z1, dz, jf1, jfn, sw,
     +		ldq, q0, ldim, otr, sa, nt, n2, idbug )
C
C  FX domain post-stack migration using implicit operator
C
C  nx,x1,dx - x grid locations
C  nz,z1,dz - z grid locations
C  jf1,jfn - frequency index range
C  sw - frequency sampling (rad/sec)
C  complex q0(ldq,*) - input wavefield at z=z1
C  ldim - leading dimension of output image array
C  otr(ldim,nx) - output array containing final image
C  complex sa(*) - work array, length 6*nx
C  nt - time length
C  n2 - fourier transform length (time)
C  idbug - debug print flag
C
	implicit none
C  Argument declarations
	integer nx,nz,jf1,jfn,ldq,ldim,nt,n2,idbug
	real x1,dx,z1,dz,sw,otr(ldim,*)
	complex q0(ldq,*), sa(*)
C  Local declarations
	integer kv, iupd, j, jf, k
	real t, frac, z, vbar, omega

C  Work array pointers
	kv = 5*nx + 1

C  initialize image time
	t = 0.0
C  up/down wave flag
	iupd = 1
C  Velocity fraction
	frac = 0.5

C  Outer loop over depth
	do k=2,nz
		z = z1 + dz*(k-1)
C  Interpolate migration velocity
		call fxmg_vmig(nx, x1, dx, z, sa(kv), vbar, sa) 
		if (idbug.ne.0 .and. mod(k-1,10).eq.0) then
			write (idbug,
     +         '('' Depth '',f8.1,'' vbar '',f8.1)') z,vbar
		endif
C  Extrapolate down one depth step for all non-zero frequencies
		do jf=max(jf1,2),jfn
			omega = sw*(jf-1)
			j = jf - jf1 + 1
			call fxmg_extrap(nx, sa(kv), q0(1,j), omega, 
     +			frac, iupd, dx, dz, sa)
			call fxmg_lens(omega, nx, q0(1,j), frac, iupd, dz, 
     +			sa(kv), vbar)
		enddo
C  Apply imaging condition
		t = t + 2.*dz/vbar
		call fxmg_image( ldq, q0, nx, jf1, jfn, sw, k, t, sa, 
     +        ldim, otr ) 
	enddo

	return
	end
C
C-----------------------------------------------------------------------
C  

	subroutine fxmg_vmig(nvx, xv1, dvx, z, vel, vbar, sa) 
C
C  Set migration velocity.  Normally, this comes from a seismic
C  data base system.  For benchmark purposes, use constant velocity.
C
C  nvx,xv1,dvx - locations where velocity is requested
C  z - depth where velocity is requested
C  vel(nvx) - output migration velocity
C  vbar - output average velocity
C  sa(nvx) - work array
C
	implicit none
	integer nvx, i
	real xv1, dvx, z, vel(nvx), vbar, sa(nvx)
	include 'fxmg.inc'

C Set constant velocity (this is where database would come in)
	do i=1,nvx
		vel(i) = vmig
	enddo

C  Compute average velocity
	vbar = 0.0
	do i=1,nvx
		vbar = vbar + vel(i)
	enddo
	vbar = vbar / float(nvx)

	return
	end
C
C-----------------------------------------------------------------------
C  
	subroutine fxmg_extrap( nx, vmig, q0, omega, frac, iupd, 
     +		dx, dz, sa )
C
C  Extrapolate wavefield one depth step for a given frequency
C
C  nx - number of horizontal locations
C  vmig(nx) - migration velocity at current depth
C  q0(nx) - wavefield at current depth and frequency
C  omega - angular frequency
C  frac - fraction of velocity to use
C  iupd - up/downgoing flag, 1=upgoing, -1=downgoing
C  dx,dz - x,z step sizes
C  sa(*) - work array, length 5*nx
C  
	implicit none
C  Arguments
	integer nx, iupd
	real vmig(nx), omega, frac, dx, dz
	complex q0(nx), sa(*)
C  Constants for tri-diagonal solver
	complex zbnx1, zbnxn, zbbnx1, zbbnxn
	integer k1, k2, k3, k4, k5, idip

C  Work array pointers
	k1 = 1
	k2 = k1 + nx
	k3 = k2 + nx
	k4 = k3 + nx
	k5 = k4 + nx

C  Initialize tri-diag constants
	idip = 65
	call fxmg_preptri( omega, vmig, sa(k1), sa(k2), 
     +	sa(k3), sa(k4),	zbnx1, zbnxn, zbbnx1, zbbnxn, 
     +	dz, nx, dx, frac, iupd, idip )

C  Call solver
	call seictri( q0, sa(k1), sa(k2), sa(k3), sa(k4),
     +	zbnx1, zbnxn, zbbnx1, zbbnxn, sa(k5), nx )

	return
	end
C
C-----------------------------------------------------------------------
C  
	subroutine fxmg_lens( omega, nx, q0, frac, iupd, dz, vmig, vbar )
C
C  Apply thin lens correction for a given depth level.  Time 
C  correction is difference between migration and average velocity
C  in the current layer:
C  dt(i) = ( dz/vmig(i) - dz/vbar )
C
C  omega - input frequency in rad/sec
C  nx - number of horizontal locations
C  q0(nx) - pressure field at omega
C  frac - velocity fraction, 1.0 = full velocity
C         Set to 0.5 for post stack case
C  iupd - up/down flag, 1=up, -1=down
C  dz - layer thickness
C  vmig(nx) - migration velocity for this depth
C  vbar - average velocity for this depth
C
	complex q0(nx)
	real omega, dz, vmig(nx), vbar

C  Loop over x location and apply phase factor
	dzf = dz/frac
	dzv = dzf/vbar
	xomega = iupd*omega
	do j=1,nx
		arg = xomega*( dzf/vmig(j) - dzv )
		q0(j) = q0(j) * cmplx( cos(arg), sin(arg) )
	enddo
	
	return
	end
C
C-----------------------------------------------------------------------
C  
	subroutine fxmg_image( ldq, q0, nx, jf1, jfn, sw, k, t, sa, 
     +    ldim, otr ) 
C
C  Apply imaging condition for downward continuation operator
C
C  complex q0(ldq,*) - wavefield
C  nx - number of x locations
C  jf1, jfn - frequency index range
C  sw - sample frequency in rad/sec
C  k - depth level
C  t - image time
C  complex sa - work array, length 2*nx
C  ldim - leading dimension of image array
C  otr(ldim,*) - output image array, level k is set
C  
	implicit none
C  Arguments
	integer ldq, nx, jf1, jfn, k, ldim
	real sw, t
	complex q0(ldq,*), sa(*)
	real otr(ldim,*)
C  Local variables
	integer i, j, jf, ntrc
	real arg
	complex wt

C  Loop over frequency and space and sum phase shifted values
	do i=1,nx
		sa(i) = (0.,0.)
	enddo
	do jf = max(2,jf1),jfn
		arg = sw*(jf-1)*t
		wt = cmplx( cos(arg), sin(arg) )
		j = jf -jf1 + 1
		do i=1,nx
			sa(i) = sa(i) + wt*q0(i,j)
		enddo
	enddo

C  Composite partial sums
	call fxmg_add( sa, sa(ldq+1), ntrc )

C  Set output values
	do i=1,ntrc
		otr(k,i) = real(sa(i))
	enddo	

	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine fxmg_add( sa, wk, ntrc )
C
C  Sum partial images across nodes
C  sa - complex input/output image array
C       On input contains partial image length nx on each node
C       On output contains composite image length jntpn decomposed 
C         across nodes
C  wk - complex work array, length nx
C  ntrc - output number of image values on this node
C
	complex sa(*), wk(*)
	include 'para.inc'
	include 'fxmg.inc'
	real t0, etimer

	ntrc = jnlive
	if (nodes.lt.2) return

C  There is now an array containing a partial image on each node.
C  Treat as a 3D array distributed over the third dimension:
C        nx/p  .  p  |  p 
C         x1     x2     f
C  where p is the number of processors. Use transpose
C  operation to convert to:
C        nx/p  .  p  |  p   ->  nx/p  .  p  |  p
C         x1     x2     f        x1      f    x2
C  After which we may sum over f:
C        nx/p  .  p  |  p   ->  nx/p  |  p
C         x1      f    x2        x1     x2
	
C  Collect partial sums across nodes
	t0 = etimer(0)
	call jtran23( 2*jntpn, sa, wk )
	tcomm = tcomm + etimer(1)

C  Sum partial images
	do j=2,nodes
		do i=1,ntrc
			sa(i) = sa(i) + sa(jntpn*(j-1)+i)
		enddo
	enddo
	
	return
	end
C
C-----------------------------------------------------------------------
C  
C    FXMG_PREPTRI:
C
C    THIS ROUTINE FOLLOWED BY SEICTRI TOGETHER PROPAGATE
C    ONE FREQUENCY VECTOR IN X THROUGH ONE Z-LEVEL. 
C    THIS DOES *NOT* HANDLE THE `THIN LENS TERM' - ONLY THE
C    `DIFFRACT TERM'.
C
C    PREPTRI SETS UP THE DIFFERENCE OPERATOR AND CTRI APPLIES IT.
C
C    COMPUTES TRIDIAG MATRIX ELEMENTS FOR LEFT AND RIGHT SIDES
C    OF LAYER DOWNWARD CONTINUATION EQUATIONS GIVEN VELOCITIES
C    AND BASIC GEOMETRY INFO.
C
C    A & B ARE ASSOCIATED WITH LEFT SIDE; AA & BB WITH RIGHT.
C
C                                                                  
C     ARRAY DIMENSION REQUIREMENT:                                
C                                                                  
C     COMPLEX                                                      
C         ZA -  NX                                                 
C         ZB -  NX                                                 
C        ZAA -  NX                                                 
C        ZBB -  NX
C  
C    COMPLEX SCALARS FOR ABSORBING BOUNDARIES
C                                                
C      ZBNX1 -  UPPER LEFT  CORNER ELEMENT, LEFT  HAND SIDE
C      ZBNXN -  LOWER RIGHT CORNER ELEMENT, LEFT  HAND SIDE
C     ZBBNX1 -  UPPER LEFT  CORNER ELEMENT, RIGHT HAND SIDE
C     ZBBNXN -  LOWER RIGHT CORNER ELEMENT, RIGHT HAND SIDE
C                                                                  
C                                                                  
C 
C
C  ON ENTRY:
C		WL   = ANGULAR FREQUENCY 		REAL, SCALAR
C		V    = VELOCITY ARRAY FOR Z-LAYER 	REAL, VECTOR
C		DZM  = DELTA-Z STEP SIZE FOR LAYER	REAL, SCALAR
C		NX   = NUMBER OF X-LOCATIONS		INT,  SCALAR
C 		DX   = DELTA-X SPACING   		REAL, SCALAR
C       FRAC = FRACTION OF VELOCITY TO USE
C              FRAC = 1.0 FOR FULL VELOCITY
C       IUPD = UP/DOWNGOING FLAG, 1=UP, -1=DOWN
C		IDIP = 65 				INT,  SCALAR
C
C  ON RETURN:
C		COMPLEX DIFFERENCE OPERATOR IS INITIALIZED FOR
C		USE IN SEICTRI.
C
C        I.E.-  ZA(NX),ZB(NX),ZAA(NX),ZBB(NX),
C	 	ZBNX1, ZBNXN, ZBBNX1, ZBBNXN 	
C		ARE RETURNED FOR THE LAYER.
C
C
      SUBROUTINE FXMG_PREPTRI(  WL,V,ZA,ZB,ZAA,ZBB,
     1 				ZBNX1,ZBNXN,ZBBNX1,ZBBNXN,
     2              DZM, NX, DX, FRAC, IUPD, IDIP  )            
C                                                                   
      COMPLEX ZA(NX),ZB(NX),ZAA(NX),ZBB(NX)                             
      COMPLEX ZBNX1,ZBNXN,ZBBNX1,ZBBNXN                             
      DIMENSION V(NX)                                                
      COMPLEX ZCC
      REAL    ETA, A, B, BETA
C                                                                   
C                                                                   
      DATA A    /0.854611587/                                       
      DATA B    /0.376369527/                                       
C                                                                   
      DATA BETA /0.101/                                             
      DATA PI   /3.14159265/
C                                                                   
C  SET SIGN OF DELTA Z AND VISCOSITY ETA BASED ON UP/DOWN FLAG
      IF (IUPD.NE.-1) THEN
          DZ = DZM
          ETA = PI
          XUPD = 1.0
      ELSE
          DZ = -DZM
          ETA = -PI
          XUPD = -1.0
      ENDIF
C
C  SET VELOCITY FRACTION
      IF (FRAC.EQ.0.) THEN
          FACT = 1.0
      ELSE
          FACT = FRAC
      ENDIF
C                                                                   
C  CONSTANTS
      DX4 = .25*DX                                                  
      DEN=WL*WL+ETA*ETA                                             
      ALPHA=DX*DX/DZ                                               
      C3R=A-B                                                       
      C2A=B/DZ*WL/DEN                                              
      C2B=B/DZ*ETA/DEN                                             
C                                                                   
C     CALCULATE THE TRI-D ELEMENTS                                  
C                                                                   
      DO IX = 1 ,NX                                             
C                                                                   
          WLDV = WL/(FACT*V(IX))
          C1I  = WLDV*ALPHA                                             
          C2R  = C2B*(FACT*V(IX))
          C2I  = C2A*(FACT*V(IX))+C1I*BETA
C                                                                   
          ZA(IX)  = CMPLX(C2R+.5*C3R,C2I)                               
          ZAA(IX) = CMPLX(C2R-.5*C3R,C2I)                               
          ZB(IX)  = CMPLX(0.,C1I)-2.*ZA(IX)                             
          ZBB(IX) = CMPLX(0.,C1I)-2.*ZAA(IX)                            
C
      ENDDO
C                                                                   
C  ABSORBING BOUNDARY CONDITIONS ********
C
        C         = XUPD * WL/(FACT*V(1))*DX4
        CC        = C*C                                             
        ZCC       = CMPLX(1.-CC,2.*C)/(1.+CC)                       
        ZBNX1     = ZB(1)+ZCC*ZA(1)                                 
        ZBBNX1    = ZBB(1)+ZCC*ZAA(1)                               
C                                                                   
        C         = XUPD * WL/(FACT*V(NX))*DX4
        CC        = C*C                                             
        ZCC       = CMPLX(1.-CC,2.*C)/(1.+CC)                       
        ZBNXN     = ZB(NX)+ZCC*ZA(NX)                               
        ZBBNXN    = ZBB(NX)+ZCC*ZAA(NX)                             
C                                                                   
C                                                                   
      RETURN                                                        
      END                                                           
C
C-----------------------------------------------------------------------
C
	subroutine fxmg_attr()
C
C  Set datacube attributes
C
	include 'group.inc'
	include 'para.inc'
	include 'fxmg.inc'

C  Panel Decomposition
	pmode = 1
	ntpn = jntpn
	nlive = jnlive

C  Data attributes
	ns = nz
	msr = nint(dz)
	ntpg = nx

	return
	end
C
C-----------------------------------------------------------------------
