C*----------------------------------------------------------------------
C*
C*  NAME:
C*    pfwrp, pfwra, pfwrb, pfwrc - Seismic Processing Benchmark 
C*        process PFWR
C*     
C*  SYNOPSIS:
C*    integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
C*    real otr(ldim,maxtrc), ra(nra), sa(nra)
C*    logical abort
C*    call pfwrp( ldim, maxtrc, nra, nsa, abort, ipr )
C*    call pfwra( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                abort, ipr )
C*    call pfwrb( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntri, ntro, abort, ipr )
C*    call pfwrc( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntro, abort, ipr )
C*
C*  DESCRIPTION: 
C*    Creates output seismic benchmark format file
C*
C*  PARAMETERS: 
C*
C*  character path - path name for output SBF dataset
C*  character dbug - Debug flag, set to 'yes' for debug output
C*  character mode - Write mode, set to 'Accum' for write and accumulate
C*  character pathfile - path name for file containing multiple paths
C*                   to use when creating SBF datasets
C*  integer nfiles - number of files to use for trace data
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*     Include files:
C*       ./coord.inc
C*       ./group.inc
C*       ./pfwr.inc
C*
C*     Libraries:
C*       libbench.a
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        12/31/91 - C. Mosher, ARCO E&PT 
C*
C*      References:
C*
C*      Modified:
C*
C*----------------------------------------------------------------------

C*========================= START OF SUBROUTINE ========================
C*   NAME:
C*     pfwrp
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   This process reads 'PFWR' cards, checks syntax, and sets
C*   memory requirements for the processing phase
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*      Subroutines:
C*        getparm - read parameter record
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/06/90 - C. Mosher
C*
C*      References:
C*        SPARC System Documentation
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine pfwrp( ldim, maxtrc, nra, nsa, abort, ipr )
	implicit none
C
C*----------------------------------------------------------------------
C*  ARGUMENTS:
C*----------------------------------------------------------------------
C
C  Output leading dimension of trace array required for the PROC step
	integer LDIM
C  Output max number of traces per group to be output during PROC step
	integer MAXTRC
C  Number of words of reserved storage required in PROC step
	integer NRA
C  Number of words of working storage required in PROC step
	integer NSA
C  Abort flag, set to .TRUE. by this process if errors detected
	logical ABORT
C  Print unit number that this routine should use
	integer IPR

C*----------------------------------------------------------------------
C*  Common blocks     
C*----------------------------------------------------------------------

	include 'coord.inc'
	include 'group.inc'
	include 'para.inc'
	include 'pfwr.inc'

C*----------------------------------------------------------------------
C*  Local declarations
C*----------------------------------------------------------------------

	integer ier, jns, jntpg, jngpl, jnlin, jfiles, ntrc
	character*80 buf
	integer sbfopen, sbfqfiles, sbfpaths

C*----------------------------------------------------------------------
C*  Start of executable statements
C*----------------------------------------------------------------------

C  Get parameter record with file name
	call sysparm( 'path', 'C', dsn, ier )
	if (ier.ne.0) then
		write (ipr,*) 'Parameter "path" required for process PFWR'
		abort = .true.
		return
	endif

C  Write with accumulation mode
	accum = .false.
	call sysparm( 'mode', 'C', buf, ier )
	if (ier.eq.0 .and. buf(1:5).eq.'Accum') accum = .true.

	nfiles = 0
	ipath = 0
	if (.not. accum) then
C  Number of files
		call sysparm( 'nfiles', 'I', nfiles, ier )
C  Path file
		call sysparm( 'pathfile', 'C', pathfile, ier )
		if (ier.eq.0 .and. pathfile(1:4).ne.'null') ipath = 1
	endif

C  Debug flag
	idbug = 0
	call sysparm( 'debug', 'C', buf, ier )
	if (ier.eq.0 .and. buf(1:3).eq.'yes') idbug = 1

C  Serial/parallel write flag
	wrpar = .true.
	call sysparm( 'serial', 'C', buf, ier )
	if (ier.eq.0 .and. buf(1:4).eq.'true') wrpar = .false.
CTR     write (ipr,*) 'serial/parallel write flag, wrpar =  ',wrpar

	write (ipr,9010) dsn,ns,ntpg,ngpl,nlin
 9010 format (' Process PFWR : Parallel Seismic Benchmark File Write',/,
     +        ' Data set name: ',a60,/,
     +        ' Data set size: ',4i6)
	if (nfiles.ne.0) write (ipr,9020) nfiles
 9020 format (' No. TRC files: ',i6)
	if (.not. wrpar) write (ipr,*)
     +	' Write will be serialized on master node'

C  Attempt to open path file
	if (ipath.ne.0) then
		ier = sbfpaths(pathfile)
		if (ier.ne.0) then
			write (ipr,*) 'PFWR: could not read pathfile ',pathfile
			abort = .true.
			return
		endif
	endif

C  If in accumulate mode, check that file exists and matches in size
	if (accum) then
		handle = sbfopen( dsn, 'rw' )
		if (handle.le.0) then
			exist = .false.
			write (ipr,*) 
     +			'PFWR Warning: file will be created in accumulate mode'
		else
			exist = .true.
			call sbfinfo( handle, jns, jntpg, jngpl, jnlin ) 
			jfiles = sbfqfiles( handle )
			if (ns.ne.jns .or. ntpg.ne.jntpg .or. ngpl.ne.jngpl
     +              .or. nlin.ne.jnlin .or. jfiles.ne.nfiles) then
			    write (ipr,*)
     +		    	'PFWR Warning: output file attribute mismatch'
			endif
			call sbfclose(handle)
		endif
	endif

C  Set memory requirements
	if (pmode.eq.0) then
		ntrc = ntpg
	else
C  In paneled mode, only need to handle number of traces per node
		ntrc = ntpn
	endif
C  Coordinates
	nsa = 6*ntrc
	if (wrpar) then
		nra = 0
C  Accumulation mode requires extra trace buffer
		if (accum) nsa = nsa + ns*ntrc
	else
C  Serial mode needs room for flags and collection buffers
		nra = nodes
		nsa = nsa + ntpg*(ldim+6)
		if (accum) nsa = nsa + ns*ntpg
	endif

	return
	end

C*========================= START OF SUBROUTINE ========================
C*   NAME:
C*     pfwra, pfwrb, pfwrc
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   Processing phase routines for reading Seismic Benchmark 
C*     Format files
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*      Subroutines:
C*        seiread - read group from SBF file
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/01/92 - C. Mosher
C*
C*      References:
C*        Seismic Benchmark Users Guide
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine pfwra( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		abort, ipr )
	implicit none
C  Arguments
	integer ldim, maxtrc, nra, nsa, ipr
	logical abort
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
C  Include blocks
	include 'coord.inc'
	include 'group.inc'
	include 'para.inc'
	include 'pfwr.inc'
C  Local variables
	integer ier
	integer sbfcreat, sbfopen, sbfpaths, sbfwinfo, jmsgrng

C  Initialize I/O timing
	etio = 0.0

C  Set magic number to indicate output data stream is available
	magic = 12345

c  counter
	ngroup = 0

C  If serialized write, allocate message flags
	if (.not. wrpar) then
		msgbase = jmsgrng( nodes )
		call vfill( 0, ra, 1, nodes )
C  Nodes other than master will not access file 
		if (node.ne.master) return
	endif

C  Open file if in accumulate mode and file exists
	if (accum .and. exist) then
		handle = sbfopen( dsn, 'rw' )
	else
C  Master node creates file
CIBM-TRM    write (ipr,*) 'ln287:Got this far in pfwr, node = ', node
		if (node.eq.master) then
			if (ipath.ne.0) then
				ier = sbfpaths(pathfile)
				if (ier.ne.0) then
					write (ipr,*) 'PFWR: could not open pathfile ',
     +					pathfile
					abort = .true.
					return
				endif
			endif
			call sbfnfiles( nfiles )
			handle = sbfcreat( dsn, ns, ntpg, ngpl, nlin )
			if (handle .eq. 0) then
				write (ipr,*) 'PFWR: could not create ',dsn
				call sbferror()
				abort = .true.
				return
			endif
C  Write aux info and close
			ier = sbfwinfo(handle, msr, gx0, gy0, gdx, gdy, grot, gtype)
			call sbfclose(handle)
		endif
C  Sync and open
CIBM-TRM            if (wrpar) write (ipr,*) 'ln311:About to jsync in pfwr'
		if (wrpar) call jsync()
		handle = sbfopen( dsn, 'rw' )
	endif

C  Check for successful open
	if (handle .eq. 0) then
		write (ipr,*) 'PFWR: could not open ',dsn
		write (ipr,*) '      node ',node
		call sbferror()
		abort = .true.
		return
	endif

	return
	end

C*----------------------------------------------------------------------
C*  The 'b' routine write data to disk
C*----------------------------------------------------------------------

	subroutine pfwrb( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntri, ntro, abort, ipr )
	implicit none
C  Arguments
	integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort
C  Common blocks
	include 'coord.inc'
	include 'group.inc'
	include 'para.inc'
	include 'pfwr.inc'
C  Local variables
	integer i,j,k,ksa,kxy,ktr,ntrc

C  Pass-through traces
	ntro = ntri

C  Set number of traces on this node
	if (pmode.eq.0) then
		ntrc = ntri
	else
		ntrc = nlive
	endif

C  Extract coordinates
	k = 1
	do j=1,ntrc
		do i=1,3
			sa(k) = sxyz(i,j)
			k = k + 1
		enddo
		do i=1,3
			sa(k) = rxyz(i,j)
			k = k + 1
		enddo
	enddo

C  Write next record
	if (wrpar .or. pmode.eq.0) then
		ksa = 6*ntrc + 1
		call pfwr_wrnext( ldim, maxtrc, ntrc, otr, 
     +	    sa(ksa), ra, abort, sa, ipr )
	else
C  Special case for serialized panel mode
		kxy = 6*ntrc + 1
		ktr = kxy + 6*ntpg
		ksa = ktr + ldim*ntpg
		call pfwr_serpan( ldim, ntrc, otr, sa, sa(ktr), sa(kxy),
     +    sa(ksa), abort, ipr )
	endif

	ngroup = ngroup + 1

	return
	end

C*----------------------------------------------------------------------
C*  Close files and quit in 'c' routine
C*----------------------------------------------------------------------
c
	subroutine pfwrc( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntro, abort, ipr )
	implicit none
	integer ldim, maxtrc, nra, nsa, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort

	include 'group.inc'
	include 'para.inc'
	include 'pfwr.inc'

	real bytesm, rate

	ntro = 0

C  Close files and clean up
	call pfwr_mclean( ra )

C  Set flag to indicate file exists
	exist = .true.

	if (node.ne.master) return

C  I/O total (Mbytes) and rate (MB/s)
	bytesm = 4.e-6 * float(ntpg) * float(ngpl*nlin) * float(ns + 6)
	if (etio.eq.0.) etio = 0.1
	rate = bytesm/etio

	call sysohdr('PFWR')
	write (ipr,9010) bytesm, etio, rate
 9010 format (/,' Process PFWR Completed',/,
     +        ' Mbytes Written   : ',f10.3,/,
     +        ' I/O Elapsed Time : ',f10.3,/,
     +        ' I/O Rate (MB/s)  : ',f10.3,/)

	return
	end

C*----------------------------------------------------------------------

	subroutine pfwr_attr( name, attr, len, ier )
C
C  Add attribute to output data stream
C  name - character string for name of attribute
C  attr - attribute data array
C  len - length of attribute in bytes.
C        Use len = lenwords * jwsize() to convert from words to bytes.
C  ier - output error code, 0=success, -1=error
C
	implicit none
	character*(*) name
	real attr(*)
	integer len, ier
	include 'pfwr.inc'
	integer sbfwratr

	ier = 0
C  Do nothing if no files are open
	if (magic.ne.12345) return
C  Write attribute data to output SBF .ATR file
	ier = sbfwratr( handle, name, attr, len )	

	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine pfwr_wrnext( ldim, maxtrc, ntri, otr, 
     +    sa, ra, abort, xyz, ipr )
C 
C  Write next frame to SBF dataset.
C  Handles both parallel and serialized reads
C
C  otr(ldim,maxtrc) - input record
C  sa - work array, length maxtrc * (ldim + 6)
C  ra(nodes) - node termination flag array
C  abort - output abort flag
C  xyz(6,maxtrc) - input coordinates
C  ipr - print unit
C
	implicit none
C  Arguments
	integer ldim, maxtrc, ntri, ipr
	real otr(ldim,maxtrc), xyz(6,maxtrc), sa(*)
	integer ra(*)
	logical abort
C  Local variables
	integer msg, inode, ntrc, len, ksa, kxy, ig, il, ibuf(3)
	integer jwsize
	real t0, etimer
	include 'group.inc'
	include 'para.inc'
	include 'pfwr.inc'

	if (idbug.ne.0) write (ipr,9010) node,igroup,iline
 9010 format (' PFWRB: node, group, line = ',3i5)

C  Write data and return if in parallel mode
	if (wrpar) then
		t0 = etimer(0)
		call pfwr_write( ldim, ntri, otr, xyz,
     +		sa, abort, ipr )
		etio = etio + etimer(1)
		return
	endif

C  Save group and line numbers on master node
	ig = igroup
	il = iline

C  Serialized write on master node

C  If not master node, send request to master and then receive
	if (node.ne.master) then
C  Send group,line,ntri as write request
		len = 3*jwsize()
		ibuf(1) = igroup
		ibuf(2) = iline
		ibuf(3) = ntri
		msg = msgbase + node
CIBM-TRM    write (ipr,*) 'ln513 worker:Send gp, ln, ntri, len = ',len
		call jsend( msg, ibuf, len, master )
C  Receive acknowledgement
CIBM-TRM    write (ipr,*) 'ln 516 worker:Rec ack, len = ',1
		call jrecv( msg, ibuf, 1 )
C  Send coords and traces
		len = 6*ntri*jwsize()
C       write (ipr,*) 'ln 519 worker:Send coords, len = ',len
		call jsend( msg, xyz, len, master )
		len = ldim*ntri*jwsize()
CIBM-TRM    write (ipr,*) 'ln522 worker:Send traces, len = ',len
		call jsend( msg, otr, len, master )
		return
	endif

C  Master node loops and processes requests
	kxy = ldim*maxtrc + 1
	ksa = kxy + 6*maxtrc
	t0 = etimer(0)
	do 10 inode = 1, nodes-1

C  Check for this node done reading
		if (ra(inode).ne.0) goto 10
C  Receive request from current node
		len = 3*jwsize()
		msg = msgbase + inode
CIBM-TRM    write (ipr,*) 'ln 539 worker:Rec, len = ',len
		call jrecv( msg, ibuf, len )
C  Check for terminate message
		if (ibuf(1).le.0) then
			ra(inode) = -1
			goto 10
		endif
C  Send acknowledgement
CIBM-TRM    write (ipr,*) 'Send ack, len = ',1
		call jsend( msg, ibuf, 1, inode )

C  Receive traces and coords
		ntrc = ibuf(3)
		len = 6*ntrc*jwsize()
CIBM-TRM    write (ipr,*) 'ln 553 worker:Rec, len = ',len
		call jrecv( msg, sa(kxy), len )
		len = ldim*ntrc*jwsize()
CIBM-TRM    write (ipr,*) 'ln 556 worker:Rec, len = ',len
		call jrecv( msg, sa, len )

C  Write data to disk
		igroup = ibuf(1)
		iline = ibuf(2)
		call pfwr_write( ldim, ntrc, sa, sa(kxy),
     +		sa(ksa), abort, ipr )
		if (abort) return

  10  continue

C  write  data for master node
	igroup = ig
	iline = il
	call pfwr_write( ldim, ntri, otr, xyz,
     +	sa, abort, ipr )
	
C  Update I/O time
	etio = etio + etimer(1)

	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine pfwr_serpan( ldim, ntri, otr, xyz, sotr, sxyz,
     +    sa, abort, ipr )
C 
C  Write next frame to SBF dataset.
C  Special case for serialized panel mode
C
C  otr(ldim,ntri) - input record panel
C  xyz(6,ntri) - input coordinates for panel
C  sotr(ldim,maxtrc) - work array to hold entire record
C  sxyz(6,maxtrc) - work array to hold coords for entire record
C  sa - work array for accumulate mode, length ns*ntpg
C  abort - output abort flag
C  ipr - print unit
C
	implicit none
C  Arguments 
	integer ldim, ntri, ipr
	real otr(ldim,ntri), xyz(6,ntri), sotr(ldim,*), sxyz(6,*), sa(*)
	logical abort
C  Local variables
	integer i, j, ier, msg, inode, ntrc, len
	integer jwsize, sbfwrite
	real t0, etimer
	include 'group.inc'
	include 'para.inc'
	include 'pfwr.inc'

	if (idbug.ne.0) write (ipr,9010) node,igroup,iline
 9010 format (' PFWRB: node, group, line = ',3i5)

C  If not master node, send request to master and then send data
	if (node.ne.master) then
C  Send number of traces as write request
		len = jwsize()
		msg = msgbase + node
CIBM-TRM    write (ipr,*) 'ln613 worker:Send ntr, len = ',len
		call jsend( msg, ntri, len, master )
C  Receive acknowledgement
CIBM-TRM    write (ipr,*) 'ln 620 worker:Rec, len = ',len
		call jrecv( msg, ier, len )
C  Send coords and traces
		len = 6*ntri*jwsize()
CIBM-TRM    write (ipr,*) 'ln619 worker:Send coords, len = ',len
		call jsend( msg, sa, len, master )
		len = ldim*ntri*jwsize()
CIBM-TRM    write (ipr,*) 'ln622 worker:Send traces, len = ',len
		call jsend( msg, otr, len, master )
		return
	endif

C  Copy master node data to panel array
	do j=1,ntri
		do i=1,ns
			sotr(i,j) = otr(i,j)	
		enddo
		do i=1,6
			sxyz(i,j) = xyz(i,j)
		enddo
	enddo

C  Master node loops and receives panels from each node
	ntrc = ntri
	t0 = etimer(0)
	do 10 inode = 1, nodes-1

C  Recive request from current node
		len = jwsize()
		msg = msgbase + inode
CIBM-TRM    write (ipr,*) 'ln 650 master:Rec, len = ',len
		call jrecv( msg, ntri, len )
C  Send acknowledgement
CIBM-TRM    write (ipr,*) 'ln 653 master:Snd, len = ',len
		call jsend( msg, ntri, len, inode )

C  Receive traces and coords
		j = ntrc + 1
		ntrc = ntrc + ntri
		len = 6*ntri*jwsize()
CIBM-TRM    write (ipr,*) 'ln 660 master:Rec, len = ',len
		call jrecv( msg, sxyz(1,j), len )
		len = ldim*ntri*jwsize()
CIBM-TRM    write (ipr,*) 'ln 663 master:Rec, len = ',len
		call jrecv( msg, sotr(1,j), len )

  10  continue

C  write data on master node
	if (accum) then
		call pfwr_accum( 0, handle, igroup, iline, 
     +		ldim, ntrc, ns, sotr, sa, ier ) 
	else
		ier = sbfwrite( handle, igroup, iline, ldim, ntrc, sotr, sxyz )
	endif
	
C  Update I/O time
	etio = etio + etimer(1)

C  Process errors
	if (ier.ne.0) then
		write (ipr,9020) node,igroup,iline,ier
 9020	format ('PFWR - Write error: node,igroup,iline,ier = ',4i5)
		call sbferror()
		abort = .true.
	endif

	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine pfwr_mclean( ra )
C
C  Nodes send termination message to master
C  Master node receives any excess termination messages
C  
	implicit none
	integer ra(*)
	integer len, inode, ibuf(3), jwsize
	include 'para.inc'
	include 'pfwr.inc'

C  If in parallel mode, close file and return
	if (wrpar) then
		call sbfclose(handle)
		return
	endif

	len = 3*jwsize()

C  Send termination message to master
	if (node.ne.master) then
		ibuf(1) = -1
		ibuf(2) = 0
		ibuf(3) = 0
		call jsend( msgbase + node, ibuf, len, master )
		return
	endif

C  Loop over nodes
	do inode = 1, nodes-1
C  Receive outstanding termination message if needed
		if (ra(inode).eq.0) then
			call jrecv( msgbase + inode, ibuf, len )
			ra(inode) = -1
		endif
	enddo

C  Close file
	call sbfclose(handle)

	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine pfwr_write( ldim, ntri, otr, xyz,
     +    sa, abort, ipr )
C
C  Write data to SBF format disk file
C
C  otr(ldim,maxtrc) - input trace data
C  xyz(6,maxtrc) - input coordinate data
C  sa - work array for accumulate mode
C  abort - abort flag
C  ipr - print unit number
C
	implicit none
C  Arguments
	integer ldim, ntri, ipr
	real otr(ldim,ntri), xyz(6,ntri), sa(*)
	logical abort
C  Local variables
	integer ntrc, ier
	integer sbfwrite, sbfwritep
	include 'group.inc'
	include 'para.inc'
	include 'pfwr.inc'
	
C  Write for accumulate mode
	if (accum .and. exist) then
		if (pmode.eq.0) then
			ntrc = ntri
		else
			ntrc = ntpn
		endif
		call pfwr_accum( pmode, handle, igroup, iline, 
     +		ldim, ntri, ns, otr, sa, ier ) 
	else
C  Write for gather mode or paneled mode
		if (pmode.eq.0) then
			ier = sbfwrite( handle, igroup, iline, 
     +     	     ldim, ntri, otr, xyz )
		else
			ier = sbfwritep( handle, igroup, iline,
     +     	     ldim, ntri, otr, xyz )
		endif
	endif
C  Process errors
	if (ier.ne.0) then
		write (ipr,9010) node,igroup,iline,ier
 9010	format ('PFWR - Write error: node,igroup,iline,ier = ',4i5)
		call sbferror()
		abort = .true.
	endif

	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine pfwr_accum( pmode, handle, igroup, iline, 
     +    ldim, ntro, ns, otr, sa, ier ) 
C
C  Write and accumulate records to PBF dataset
C  pmode - parallel mode
C  handle - PBF file handle
C  igroup - group number
C  iline - line number
C  ldim,ntro - trace array dimensions
C  ns - number of live samples per trace
C  otr - input trace array
C  sa - work array, length ns*ntro
C  ier - output error code, zero for success, negative on errors
C
	implicit none
	integer pmode, handle, igroup, iline, ldim, ntro, ns, ier
	real otr(ldim,ntro), sa(*)
	integer sbfrdfrm, sbfwrfrm, sbfrdfrmp, sbfwrfrmp
	integer i,j,k

C  Read frame from output data set
	call sbfsetvol( handle, iline )
	if (pmode.eq.0) then
		ier = sbfrdfrm( handle, igroup, sa )
	else
		ier = sbfrdfrmp( handle, igroup, sa )
	endif
	if (ier.ne.0) return

C  Accumulate current record
	k = 0
	do j=1,ntro
		do i=1,ns
			k = k + 1
			sa(k) = sa(k) + otr(i,j)
		enddo
	enddo

C  Write record back out
	if (pmode.eq.0) then
		ier = sbfwrfrm( handle, igroup, sa )
	else
		ier = sbfwrfrmp( handle, igroup, sa )
	endif

	return
	end
