C*----------------------------------------------------------------------
C*
C*  NAME:
C*    ratep, ratea, rateb, ratec - Seismic Processing Benchmark 
C*        process RATE
C*     
C*  SYNOPSIS:
C*    integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
C*    real otr(ldim,maxtrc), ra(nra), sa(nra)
C*    logical abort
C*    call ratep( ldim, maxtrc, nra, nsa, abort, ipr )
C*    call ratea( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                abort, ipr )
C*    call rateb( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntri, ntro, abort, ipr )
C*    call ratec( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntro, abort, ipr )
C*
C*  DESCRIPTION: 
C*    Compute average amplitude profile, used as checksum for
C*      benchmark verification.
C*
C*  PARAMETERS: 
C*    Record 1
C*    Field  Columns   Description
C*        1  10-70     Comment for amplitude printout
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*     Include files:
C*       ./group.inc
C*       ./para.inc
C*       ./rate.inc
C*
C*     Libraries:
C*       libbench.a
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        12/31/91 - C. Mosher, ARCO E&PT 
C*
C*      References:
C*
C*      Modified:
C*
C*----------------------------------------------------------------------

C*========================= START OF SUBROUTINE ========================
C*   NAME:
C*     ratep
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   This computes an average amplitude profile for a (5,11) array
C*   of points in each record and prints the table at the end of
C*   processing.  Should yield the same results independent of 
C*   number of processors used.
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*      Subroutines:
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/06/90 - C. Mosher
C*
C*      References:
C*        Seis user documentation
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine ratep( ldim, maxtrc, nra, nsa, abort, ipr )
	implicit none
C
C*----------------------------------------------------------------------
C*  ARGUMENTS:
C*----------------------------------------------------------------------
C
C  Output leading dimension of trace array required for the PROC step
	integer LDIM
C  Output max number of traces per group to be output during PROC step
	integer MAXTRC
C  Number of words of reserved storage required in PROC step
	integer NRA
C  Number of words of working storage required in PROC step
	integer NSA
C  Abort flag, set to .TRUE. by this process if errors detected
	logical ABORT
C  Print unit number that this routine should use
	integer IPR

C*----------------------------------------------------------------------
C*  Common blocks     
C*----------------------------------------------------------------------

	include 'group.inc'
	include 'para.inc'
	include 'rate.inc'

C*----------------------------------------------------------------------
C*  Local declarations
C*----------------------------------------------------------------------
	integer index, ier, jproc, jproc1, jprocn
	character*80 buf

C*----------------------------------------------------------------------
C*  Start of executable statements
C*----------------------------------------------------------------------

C  Set memory requirements
	ldim = ns
	maxtrc = ntpg
	nra = 0
	call rate_nproc( nproc )
	ksa = nodes*nproc + 1
	nsa = 2 * nodes * nproc

C  Check to be sure this is the first occurrence of 'RATE'
	call syspocr( 'RATE', jproc, jproc1, jprocn )
	if (jproc.ne.jproc1) return

	write (ipr,9010) 
 9010 format (' Process RATE : Measure processing rate')

C  Get parameter record with table comment
	index = 1
	call getparm('RATE',index,buf,ier)
	if (ier.eq.0) then
		com = buf(11:74)
		write (ipr,9020) com
 9020 	format (' Comment      : ',a64)
	else
		com(1:4) = '    '
	endif

C  Store data size going into flow
	ngroup = ngpl*nlin
	bytesm = 4.e-6 * float(ngroup) * float(ntpg*ns)

	return
	end

C*========================= START OF SUBROUTINE ========================
C*   NAME:
C*     ratea, rateb, ratec
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   Processing phase routines for ARCO Seismic Benchmark 
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/01/92 - C. Mosher
C*
C*      References:
C*        Seismic Benchmark Users Guide
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine ratea( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		abort, ipr )
	implicit none
C  Arguments
	integer ldim, maxtrc, nra, nsa, ipr
	logical abort
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
C  Common blocks
	include 'group.inc'
	include 'para.inc'
	include 'rate.inc'
C Local variables
	integer jproc, jproc1, jprocn
	real etimer

C  Check to be sure this is the first occurrence of 'RATE'
	call syspocr( 'RATE', jproc, jproc1, jprocn )
	if (jproc.ne.jproc1) return

c  Initialize elapsed time
	call jsync()
	et = etimer(0)

	return
	end

C*----------------------------------------------------------------------
C*  The 'b' does nothing
C*----------------------------------------------------------------------

	subroutine rateb( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntri, ntro, abort, ipr )
	implicit none
C  Arguments
	integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort

	ntro = ntri

	return
	end

C*----------------------------------------------------------------------
C*  Print table at end of processing
C*----------------------------------------------------------------------
c
	subroutine ratec( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntro, abort, ipr )
	implicit none
C  Arguments
	integer ldim, maxtrc, nra, nsa, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort
C  Common blocks
	include 'group.inc'
	include 'para.inc'
	include 'rate.inc'
C  Local variables
	integer jproc, jproc1, jprocn
	real etimer, tput, prate

	ntro = 0

C  Make sure this is the last occurrence of RATE
	call syspocr( 'RATE', jproc, jproc1, jprocn )
	if (jproc.ne.jprocn) return

C  Sync and terminate timing
	call jsync()
	et = etimer(0) - et
	if (et.eq.0.) et = 1.

C  Get per process times from each node
	call rate_collect( nproc, nodes, sa, sa(ksa) )
	if (node.ne.master) return

C  Print header lines and comment
	call sysohdr('RATE')
	write (ipr,*) ' Processing Rate Measurement'
	if (com(1:4).ne.'    ') write (ipr,*) ' Comment: ',com

C  Print throughput (MB/s) and processing rate (groups/hour)
	tput = bytesm/et
	prate = 3600.*float(ngroup)/et

	write (ipr,9020) ngroup, bytesm, et, tput, prate
 9020 format (' Groups processed : ',i8,/,
     +        ' Mbytes processed : ',f10.2,/,
     +        ' Elapsed Time     : ',f10.2,/,
     +        ' Data Throughput  : ',g12.4,' Mb/s',/,
     +        ' Processing Rate  : ',g12.4,' Groups/hr')

C  Print table of per-process times
	call rate_print( nproc, nodes, sa, sa(ksa) )

	return
	end

C*----------------------------------------------------------------------

	subroutine rate_nproc( nprc )
C
C  Return number of processes from PROC card
C
	integer nprc
	include 'sys.inc'

	nprc = nproc

	return
	end

C*----------------------------------------------------------------------

	subroutine rate_collect( n1, n2, ct, et )
C
C  Collect cpu and elapsed times
C
	implicit none
	integer n1, n2
	real ct(n1,n2), et(n1,n2)
	include 'para.inc'
	include 'sys.inc'
	integer j,msg,len,jwsize

C  Length in bytes of time arrays
	len = n1 * jwsize()

C  Master node collects
	if (node.eq.master) then
		call scopy( nproc, ctime, 1, ct, 1 )
		call scopy( nproc, etime, 1, et, 1 )
		do j=2,nodes
			msg = j-1
			call jrecv( msg, ct(1,j), len )
			call jrecv( msg, et(1,j), len )
		enddo
	else
C  Other nodes send to master
		call jsend( node, ctime, len, master )
		call jsend( node, etime, len, master )
	endif

	return
	end

C*----------------------------------------------------------------------

	subroutine rate_print( n1, n2, ct, et )
C
C  Print per-node cpu and elapsed times
C
	implicit none
	integer n1, n2
	real ct(n1,n2), et(n1,n2)
	include 'para.inc'
	include 'sys.inc'
	integer i,j

	write (ipr,'(/,'' Per node CPU times (sec): '')')
	do i=1,nproc
		write (ipr,9010) name(i),(nint(ct(i,j)),j=1,nodes)
 9010 	format (1x,a4,':',8i8,/,(6x,8i8))
	enddo
	write (ipr,'(/,'' Per node Elapsed times: '')')
	do i=1,nproc
		write (ipr,9010) name(i),(nint(et(i,j)),j=1,nodes)
	enddo

	return
	end
