C*----------------------------------------------------------------------
C*
C*  NAME:
C*    tr23p, tr23a, tr23b, tr23c - Seismic Processing Benchmark 
C*        process TR23
C*     
C*  SYNOPSIS:
C*    integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
C*    real otr(ldim,maxtrc), ra(nra), sa(nra)
C*    logical abort
C*    call tr23p( ldim, maxtrc, nra, nsa, abort, ipr )
C*    call tr23a( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                abort, ipr )
C*    call tr23b( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntri, ntro, abort, ipr )
C*    call tr23c( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntro, abort, ipr )
C*
C*  DESCRIPTION: 
C*    Transpose second two dimensions of seismic data stream
C*
C*  PARAMETERS: 
C*
C*  character path - path name for temp file during transpose
C*  integer nfiles - number of files to use for temp dataset
C*  character pathfile - pathfile for temp dataset
C*  character dbug - Debug flag, set to 'yes' for debug output
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*     Include files:
C*       ./coord.inc
C*       ./group.inc
C*       ./tr23.inc
C*
C*     Libraries:
C*       libbench.a
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        12/31/91 - C. Mosher, ARCO E&PT 
C*
C*      References:
C*
C*      Modified:
C*
C*----------------------------------------------------------------------

C*========================= START OF SUBROUTINE ========================
C*   NAME:
C*     tr23p
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   This process reads 'TR23' cards, checks syntax, and sets
C*   memory requirements for the processing phase
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*      Subroutines:
C*        sysparm - read parameter record
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/06/90 - C. Mosher
C*
C*      References:
C*        SPARC System Documentation
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine tr23p( ldim, maxtrc, nra, nsa, abort, ipr )
	implicit none
C
C*----------------------------------------------------------------------
C*  ARGUMENTS:
C*----------------------------------------------------------------------
C
C  Output leading dimension of trace array required for the PROC step
	integer LDIM
C  Output max number of traces per group to be output during PROC step
	integer MAXTRC
C  Number of words of reserved storage required in PROC step
	integer NRA
C  Number of words of working storage required in PROC step
	integer NSA
C  Abort flag, set to .TRUE. by this process if errors detected
	logical ABORT
C  Print unit number that this routine should use
	integer IPR

C*----------------------------------------------------------------------
C*  Common blocks     
C*----------------------------------------------------------------------

	include 'coord.inc'
	include 'group.inc'
	include 'para.inc'
	include 'tr23.inc'

C*----------------------------------------------------------------------
C*  Local declarations
C*----------------------------------------------------------------------

	integer ier
	character*80 buf

C*----------------------------------------------------------------------
C*  Start of executable statements
C*----------------------------------------------------------------------

C  Get parameter record with file name
	call sysparm( 'path', 'C', path, ier )
	if (ier.ne.0) then
		write (ipr,*) 'Parameter "path" required for process TR23'
		abort = .true.
		return
	endif

C  Optional multi-file parameters
C  Number of files
	nfiles = 0
	call sysparm( 'nfiles', 'I', nfiles, ier )
C  Path file
	ipath = 0
	call sysparm( 'pathfile', 'C', pathfile, ier )
	if (ier.eq.0 .and. pathfile(1:4).ne.'null') ipath = 1

C  Optional delete flag
	delete = .true.
	call sysparm( 'delete', 'C', buf, ier )
	if (ier.eq.0 .and. buf(1:2).eq.'no') delete = .false.

C  Debug flag
	idbug = 0
	call sysparm( 'debug', 'C', buf, ier )
	if (ier.eq.0 .and. buf(1:3).eq.'yes') idbug = 1

	jns = ns
	jntpg = ngpl
	jngpl = ntpg
	jnlin = nlin
	jpmode = pmode

	write (ipr,9010) path,ns,ntpg,ngpl,nlin,jns,jntpg,jngpl,jnlin
 9010 format (' Process TR23 : Transpose Seismic Data Stream ',/,
     +        ' Data set name: ',a60,/,
     +        ' Input size   : ',4i6,/,
     +        ' Output size  : ',4i6)

	if (nfiles.ne.0) write (ipr,'('' No. of files : '',i6)') nfiles
	if (ipath.ne.0) write (ipr,*) 'Path file    : ',pathfile

C  Set memory requirements
	ldim = jns
	maxtrc = jntpg
	nra = 0
	nsa = 6*max(ntpg,ngpl)
	ksa = nsa + 1
	nsa = nsa + jns*jntpg

C  Set output attributes
	call tr23atr()

	return
	end

C*========================= START OF SUBROUTINE ========================
C*   NAME:
C*     tr23a, tr23b, tr23c
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   Processing phase routines for reading Seismic Benchmark 
C*     Format files
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*      Subroutines:
C*        seiread - read group from SBF file
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/01/92 - C. Mosher
C*
C*      References:
C*        Seismic Benchmark Users Guide
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine tr23a( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		abort, ipr )
	implicit none
C  Arguments
	integer ldim, maxtrc, nra, nsa, ipr
	logical abort
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
C  Include blocks
	include 'coord.inc'
	include 'group.inc'
	include 'para.inc'
	include 'tr23.inc'
C  Local variables
	integer sbfcreat, sbfopen

C  Master node creates file
	if (node.eq.master) then
C  Set multi-file attributes, if defined
		call sbfnfiles( nfiles )
		if (ipath.ne.0) call sbfpaths(pathfile)
C  Create file
		handle = sbfcreat( path, ns, ntpg, ngpl, nlin )
		if (handle .eq. 0) then
			write (ipr,*) 'TR23: could not create temp file ',path
			call sbferror()
			write (*,*) 'handle = ',handle
			abort = .true.
			return
		endif
C  Close and sync
		call sbfclose( handle )
	endif
	call jsync()

C  Everybody opens
	handle = sbfopen( path, 'rw' )

C  Initialize I/O timing
	etiow = 0.0
	etior = 0.0

C  Set output attributes
	call tr23atr()

C  Set output init flag
	init = .true.

	return
	end

C*----------------------------------------------------------------------
C*  The 'b' routine writes data to disk
C*----------------------------------------------------------------------

	subroutine tr23b( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntri, ntro, abort, ipr )
	implicit none
C  Arguments
	integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort
C  Common blocks
	include 'coord.inc'
	include 'group.inc'
	include 'para.inc'
	include 'tr23.inc'
C  Local variables
	integer i,j,k,ier
	real t0, etimer
	integer sbfwrite, sbfwritep

C  No output traces from this entry
	ntro = 0

C  Extract coordinates
	k = 1
	do j=1,ntri
		do i=1,3
			sa(k) = sxyz(i,j)
			k = k + 1
		enddo
		do i=1,3
			sa(k) = rxyz(i,j)
			k = k + 1
		enddo
	enddo

	if (idbug.ne.0) write (*,
     +    '('' TR23B: write node, group, line '',3i5)') 
     +         node,igroup,iline

	ier = 0

	t0 = etimer(0)
	if (pmode.eq.0) then
C  Write for gather mode
		ier = sbfwrite( handle, igroup, iline, 
     +          ldim, ntri, otr, sa )
	else
C  Write for paneled mode
		ier = sbfwritep( handle, igroup, iline,
     +          ldim, ntri, otr, sa )
	endif
	etiow = etiow + etimer(1)

	if (ier.ne.0) then
		write (ipr,*) 'TR23: error on group, line ',igroup,iline
		call sbferror()
		abort = .true.
		return
	endif

	return
	end

C*----------------------------------------------------------------------
C*  Data is read back transposed in 'c' routine
C*----------------------------------------------------------------------
c
	subroutine tr23c( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntro, abort, ipr )
	implicit none
C  Arguments
	integer ldim, maxtrc, nra, nsa, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort
C  Common blocks
	include 'group.inc'
	include 'coord.inc'
	include 'para.inc'
	include 'tr23.inc'
C  Local declarations
	integer sbfread, sbfreadt, sbfclose
	integer i,j,k,ier
	real t0,etimer,bytesm,rater,ratew

C  First time in, initialize for reading data back from disk
	if (init) then
		init = .false.
		call jsync()
		call sysmsg('TR23: temp file write complete')
C  Set output decomposition
		call jdcomp( jngpl, jngpn, jnlive )
		call jrange( jngpl, jgrp1, jgrpn )
		if (jgrp1.eq.0) jgrp1 = 1
		jgrpi = 1
		jgroup = jgrp1 - jgrpi
		jline = 1
	endif

c  Increment group counter
	jgroup = jgroup + jgrpi

c  check for line increment
	if (jgroup.gt.jgrpn) then
		jline = jline + 1
C  Check for end of data set
		if (jline.gt.jnlin) then
			ntro = 0
C  Close file and sync
			ier = sbfclose( handle )
			call jsync()
C  Read is complete, master node outputs totals
			if (node.ne.master) return
C  I/O total (Mbytes) and rates (MB/s)
			bytesm = 4.e-6 * float(jntpg*jngpl*jnlin)*float(jns+6)
			if (etior.eq.0.) etior = 1.
			if (etiow.eq.0.) etiow = 1.
			rater = bytesm/etior
			ratew = bytesm/etiow
			write (ipr,9010) bytesm, etiow, ratew, etior, rater
 9010 format (//,' Process TR23 Completed, ',/,
     +        ' Total Mbytes    : ',f10.3,/,
     +        ' Write I/O time  : ',f10.3,/,
     +        ' Write I/O Rate  : ',f10.3,/,
     +        ' Trans I/O Time  : ',f10.3,/,
     +        ' Trans I/O Rate  : ',f10.3,/)
C  Remove temp file and return
			if (delete) then
				call sbfnfiles( nfiles )
				call sbfpaths( pathfile )
				call sbfunlink( path )	
			endif
			return
		endif
C  New line, reset group counter
		jgroup = jgrp1
c  Sync if in block mode so each line starts at same time
		call jsync()
	endif

C  Set attributes
	call tr23atr()
	ntro = ntpg
	igroup = jgroup
	iline = jline

	ier = 0

c  Read next group
	if (idbug.ne.0) write (ipr,9020) node,jgroup,jline
 9020 format (' TR23C: read node, group, line = ',3i5)
	t0 = etimer(0)
	if (jpmode.eq.0) then
		ier = sbfreadt( handle, jgroup, jline, ldim, maxtrc, otr, sa )
	else
		ier = sbfread( handle, jgroup, jline, ldim, maxtrc, otr, sa )
	endif
C  Update I/O time
	etior = etior + etimer(1)
	if (ier.ne.0) then
		write (ipr,9030) node,jgroup,jline,ier
 9030   format (' TR23 ERROR - node ',i5,' group ',i5,
     +          ' line ',i5,' ier ',i5)
		call sbferror()
		abort = .true.
		return
	endif

C  Set coordinates
	k = 1
	do j=1,ntro
		do i=1,3
			sxyz(i,j) = sa(k)
			k = k + 1
		enddo
		do i=1,3
			rxyz(i,j) = sa(k)
			k = k + 1
		enddo
	enddo

	return
	end

C*----------------------------------------------------------------------

	subroutine tr23atr()
C
C  Set attributes in system commons
C
	implicit none
	include 'coord.inc'
	include 'group.inc'
	include 'para.inc'
	include 'tr23.inc'

C  Set data set attributes
	ns = jns
	msr = jmsr
	ntpg = jntpg
	ngpl = jngpl
	nlin = jnlin

C  Parallel decomposition attributes
	pmode = 0
	ntpn = ntpg
	nlive = ntpg

	return
	end
